@@ -1,7 +1,8 @@
 """Enums used by sqlfluff."""
+
 from enum import Enum
 
-from colorama import Fore, Style
+from colorama import Back, Fore, Style
 
 
 class FormatType(Enum):
@@ -11,6 +12,8 @@ class FormatType(Enum):
     json = "json"
     yaml = "yaml"
     github_annotation = "github-annotation"
+    github_annotation_native = "github-annotation-native"
+    none = "none"  # An option to return _no output_.
 
 
 class Color(Enum):
@@ -19,4 +22,4 @@ class Color(Enum):
     red = Fore.RED
     green = Fore.GREEN
     blue = Fore.BLUE
-    lightgrey = Fore.BLACK + Style.BRIGHT
+    lightgrey = Fore.BLACK + Back.WHITE + Style.BRIGHT
