@@ -30,8 +30,9 @@ from sqlfluff.core.errors import (
     SQLLintError,
     SQLParseError,
 )
-from sqlfluff.core.file_helpers import get_encoding
+from sqlfluff.core.helpers.file import get_encoding
 from sqlfluff.core.linter.common import ParsedString, RenderedFile, RuleTuple
+from sqlfluff.core.linter.fix import apply_fixes, compute_anchor_edit_info
 from sqlfluff.core.linter.linted_dir import LintedDir
 from sqlfluff.core.linter.linted_file import (
     TMP_PRS_ERROR_TYPES,
@@ -39,10 +40,10 @@ from sqlfluff.core.linter.linted_file import (
     LintedFile,
 )
 from sqlfluff.core.linter.linting_result import LintingResult
-from sqlfluff.core.linter.noqa import IgnoreMask
 from sqlfluff.core.parser import Lexer, Parser
 from sqlfluff.core.parser.segments.base import BaseSegment, SourceFix
 from sqlfluff.core.rules import BaseRule, RulePack, get_ruleset
+from sqlfluff.core.rules.noqa import IgnoreMask
 
 if TYPE_CHECKING:  # pragma: no cover
     from sqlfluff.core.parser.segments.meta import MetaSegment
@@ -116,7 +117,8 @@ class Linter:
     ) -> Tuple[str, FluffConfig, str]:
         """Load a raw file and the associated config."""
         file_config = root_config.make_child_from_path(fname)
-        encoding = get_encoding(fname=fname, config=file_config)
+        config_encoding: str = file_config.get("encoding", default="autodetect")
+        encoding = get_encoding(fname=fname, config_encoding=config_encoding)
         # Check file size before loading.
         limit = file_config.get("large_file_skip_byte_limit")
         if limit:
@@ -181,10 +183,7 @@ class Linter:
         templating_blocks_indent = bool(templating_blocks_indent)
         # If we're forcing it through we don't check.
         if templating_blocks_indent and not force_block_indent:
-            indent_balance = sum(
-                getattr(elem, "indent_val", 0)
-                for elem in cast(Tuple[BaseSegment, ...], tokens)
-            )
+            indent_balance = sum(getattr(elem, "indent_val", 0) for elem in tokens)
             if indent_balance != 0:  # pragma: no cover
                 linter_logger.debug(
                     "Indent balance test failed for %r. Template indents will not be "
@@ -197,7 +196,7 @@ class Linter:
         # The file will have been lexed without config, so check all indents
         # are enabled.
         new_tokens = []
-        for token in cast(Tuple[BaseSegment, ...], tokens):
+        for token in tokens:
             if token.is_meta:
                 token = cast("MetaSegment", token)
                 if token.indent_val != 0:
@@ -249,9 +248,11 @@ class Linter:
                     "Line {0[0]}, Position {0[1]}: Found unparsable section: "
                     "{1!r}".format(
                         unparsable.pos_marker.working_loc,
-                        unparsable.raw
-                        if len(unparsable.raw) < 40
-                        else unparsable.raw[:40] + "...",
+                        (
+                            unparsable.raw
+                            if len(unparsable.raw) < 40
+                            else unparsable.raw[:40] + "..."
+                        ),
                     ),
                     segment=unparsable,
                 )
@@ -411,7 +412,8 @@ class Linter:
                 # Additional newlines are to assist in scanning linting loops
                 # during debugging.
                 linter_logger.info(
-                    f"\n\nEntering linter phase {phase}, loop {loop+1}/{loop_limit}\n"
+                    f"\n\nEntering linter phase {phase}, "
+                    f"loop {loop + 1}/{loop_limit}\n"
                 )
                 changed = False
 
@@ -462,7 +464,7 @@ class Linter:
                     if fix and fixes:
                         linter_logger.info(f"Applying Fixes [{crawler.code}]: {fixes}")
                         # Do some sanity checks on the fixes before applying.
-                        anchor_info = BaseSegment.compute_anchor_edit_info(fixes)
+                        anchor_info = compute_anchor_edit_info(fixes)
                         if any(
                             not info.is_valid for info in anchor_info.values()
                         ):  # pragma: no cover
@@ -490,8 +492,11 @@ class Linter:
                             # This is the happy path. We have fixes, now we want to
                             # apply them.
                             last_fixes = fixes
-                            new_tree, _, _ = tree.apply_fixes(
-                                config.get("dialect_obj"), crawler.code, anchor_info
+                            new_tree, _, _, _valid = apply_fixes(
+                                tree,
+                                config.get("dialect_obj"),
+                                crawler.code,
+                                anchor_info,
                             )
                             # Check for infinite loops. We use a combination of the
                             # fixed templated file and the list of source fixes to
@@ -500,7 +505,16 @@ class Linter:
                                 new_tree.raw,
                                 tuple(new_tree.source_fixes),
                             )
-                            if loop_check_tuple not in previous_versions:
+                            if not _valid:
+                                # The fixes result in an invalid file. Don't apply
+                                # the fix and skip onward. Show a warning.
+                                linter_logger.warning(
+                                    f"Fixes for {crawler.code} not applied, as it "
+                                    "would result in an unparsable file. Please "
+                                    "report this as a bug with a minimal query "
+                                    "which demonstrates this warning."
+                                )
+                            elif loop_check_tuple not in previous_versions:
                                 # We've not seen this version of the file so
                                 # far. Continue.
                                 tree = new_tree
@@ -1016,6 +1030,7 @@ class Linter:
         apply_fixes: bool = False,
         fixed_file_suffix: str = "",
         fix_even_unparsable: bool = False,
+        retain_files: bool = True,
     ) -> LintingResult:
         """Lint an iterable of paths."""
         # If no paths specified - assume local
@@ -1027,7 +1042,7 @@ class Linter:
         expanded_paths: List[str] = []
         expanded_path_to_linted_dir = {}
         for path in paths:
-            linted_dir = LintedDir(path)
+            linted_dir = LintedDir(path, retain_files=retain_files)
             result.add(linted_dir)
             for fname in self.paths_from_path(
                 path,
