@@ -8,6 +8,7 @@ import regex
 
 from sqlfluff.core.config import FluffConfig
 from sqlfluff.core.errors import SQLLexError
+from sqlfluff.core.helpers.slice import is_zero_slice, offset_slice, to_tuple
 from sqlfluff.core.parser.markers import PositionMarker
 from sqlfluff.core.parser.segments import (
     BaseSegment,
@@ -20,7 +21,6 @@ from sqlfluff.core.parser.segments import (
     TemplateSegment,
     UnlexableSegment,
 )
-from sqlfluff.core.slice_helpers import is_zero_slice, offset_slice, to_tuple
 from sqlfluff.core.templaters import TemplatedFile
 from sqlfluff.core.templaters.base import TemplatedFileSlice
 
@@ -283,8 +283,26 @@ class StringLexer:
             return LexMatch(forward_string, [])
 
     def construct_segment(self, raw: str, pos_marker: PositionMarker) -> RawSegment:
-        """Construct a segment using the given class a properties."""
-        return self.segment_class(raw=raw, pos_marker=pos_marker, **self.segment_kwargs)
+        """Construct a segment using the given class a properties.
+
+        Unless an override `type` is provided in the `segment_kwargs`,
+        it is assumed that the `name` of the lexer is designated as the
+        intended `type` of the segment.
+        """
+        # NOTE: Using a private attribute here feels a bit wrong.
+        _segment_class_types = self.segment_class._class_types
+        _kwargs = self.segment_kwargs
+        assert not (
+            "type" in _kwargs and "instance_types" in _kwargs
+        ), f"Cannot set both `type` and `instance_types` in segment kwargs: {_kwargs}"
+        if "type" in _kwargs:
+            # TODO: At some point we should probably deprecate this API and only
+            # allow setting `instance_types`.
+            assert _kwargs["type"]
+            _kwargs["instance_types"] = (_kwargs.pop("type"),)
+        elif "instance_types" not in _kwargs and self.name not in _segment_class_types:
+            _kwargs["instance_types"] = (self.name,)
+        return self.segment_class(raw=raw, pos_marker=pos_marker, **_kwargs)
 
 
 class RegexLexer(StringLexer):
@@ -624,7 +642,7 @@ def _iter_segments(
                             )
                         continue
 
-            elif tfs.slice_type in ("templated", "block_start"):
+            elif tfs.slice_type in ("templated", "block_start", "escaped"):
                 # Found a templated slice. Does it have length in the templated file?
                 # If it doesn't, then we'll pick it up next.
                 if not is_zero_slice(tfs.templated_slice):
@@ -794,9 +812,11 @@ class Lexer:
         # Add an end of file marker
         segment_buffer.append(
             EndOfFile(
-                pos_marker=segment_buffer[-1].pos_marker.end_point_marker()
-                if segment_buffer
-                else PositionMarker.from_point(0, 0, templated_file)
+                pos_marker=(
+                    segment_buffer[-1].pos_marker.end_point_marker()
+                    if segment_buffer
+                    else PositionMarker.from_point(0, 0, templated_file)
+                )
             )
         )
         # Convert to tuple before return
