@@ -4,8 +4,8 @@ This is designed to be the root segment, without
 any children, and the output of the lexer.
 """
 
-from typing import Any, List, Optional, Set, Tuple
-from uuid import UUID, uuid4
+from typing import Any, FrozenSet, List, Optional, Tuple
+from uuid import uuid4
 
 from sqlfluff.core.parser.markers import PositionMarker
 from sqlfluff.core.parser.segments.base import BaseSegment, SourceFix
@@ -26,11 +26,15 @@ class RawSegment(BaseSegment):
         self,
         raw: Optional[str] = None,
         pos_marker: Optional[PositionMarker] = None,
+        # For legacy and syntactic sugar we allow the simple
+        # `type` argument here, but for more precise inheritance
+        # we suggest using the `instance_types` option.
         type: Optional[str] = None,
+        instance_types: Tuple[str, ...] = (),
         trim_start: Optional[Tuple[str, ...]] = None,
         trim_chars: Optional[Tuple[str, ...]] = None,
         source_fixes: Optional[List[SourceFix]] = None,
-        uuid: Optional[UUID] = None,
+        uuid: Optional[int] = None,
     ):
         """Initialise raw segment.
 
@@ -49,17 +53,19 @@ class RawSegment(BaseSegment):
         self.pos_marker: PositionMarker = pos_marker  # type: ignore
         # Set the segments attribute to be an empty tuple.
         self.segments = ()
-        # if a surrogate type is provided, store it for later.
-        self._surrogate_type = type
+        self.instance_types: Tuple[str, ...]
+        if type:
+            assert not instance_types, "Cannot set `type` and `instance_types`."
+            self.instance_types = (type,)
+        else:
+            self.instance_types = instance_types
         # What should we trim off the ends to get to content
         self.trim_start = trim_start
         self.trim_chars = trim_chars
-        # A cache variable for expandable
-        self._is_expandable = None
         # Keep track of any source fixes
         self._source_fixes = source_fixes
-        # UUID for matching
-        self.uuid = uuid or uuid4()
+        # UUID for matching (the int attribute of it)
+        self.uuid = uuid or uuid4().int
         self.representation = "<{}: ({}) {!r}>".format(
             self.__class__.__name__, self.pos_marker, self.raw
         )
@@ -76,16 +82,6 @@ class RawSegment(BaseSegment):
 
     # ################ PUBLIC PROPERTIES
 
-    @property
-    def matched_length(self) -> int:
-        """Return the length of the segment in characters."""
-        return len(self._raw)
-
-    @property
-    def is_expandable(self) -> bool:
-        """Return true if it is meaningful to call `expand` on this segment."""
-        return False
-
     @property
     def is_code(self) -> bool:
         """Return True if this segment is code."""
@@ -117,14 +113,12 @@ class RawSegment(BaseSegment):
         return [self]
 
     @property
-    def class_types(self) -> Set[str]:
+    def class_types(self) -> FrozenSet[str]:
         """The set of full types for this segment, including inherited.
 
         Add the surrogate type for raw segments.
         """
-        return (
-            {self._surrogate_type} if self._surrogate_type else set()
-        ) | super().class_types
+        return frozenset(self.instance_types) | super().class_types
 
     @property
     def source_fixes(self) -> List[SourceFix]:
@@ -139,11 +133,13 @@ class RawSegment(BaseSegment):
 
     def get_type(self) -> str:
         """Returns the type of this segment as a string."""
-        return self._surrogate_type or self.type
+        if self.instance_types:
+            return self.instance_types[0]
+        return super().get_type()
 
     def is_type(self, *seg_type: str) -> bool:
         """Extend the parent class method with the surrogate types."""
-        if self._surrogate_type and self._surrogate_type in seg_type:
+        if set(self.instance_types).intersection(seg_type):
             return True
         return self.class_is_type(*seg_type)
 
@@ -152,7 +148,11 @@ class RawSegment(BaseSegment):
         return [self]
 
     def raw_trimmed(self) -> str:
-        """Return a trimmed version of the raw content."""
+        """Return a trimmed version of the raw content.
+
+        Returns:
+            str: The trimmed version of the raw content.
+        """
         raw_buff = self.raw
         if self.trim_start:
             for seq in self.trim_start:
@@ -174,7 +174,17 @@ class RawSegment(BaseSegment):
     def stringify(
         self, ident: int = 0, tabsize: int = 4, code_only: bool = False
     ) -> str:
-        """Use indentation to render this segment and its children as a string."""
+        """Use indentation to render this segment and its children as a string.
+
+        Args:
+            ident (int, optional): The indentation level. Defaults to 0.
+            tabsize (int, optional): The size of each tab. Defaults to 4.
+            code_only (bool, optional): Whether to render only the code.
+                Defaults to False.
+
+        Returns:
+            str: The rendered string.
+        """
         preface = self._preface(ident=ident, tabsize=tabsize)
         return preface + "\n"
 
@@ -182,6 +192,9 @@ class RawSegment(BaseSegment):
         """Return any extra output required at the end when logging.
 
         NB Override this for specific subclasses if we want extra output.
+
+        Returns:
+            str: The extra output.
         """
         return f"{self.raw!r}"
 
@@ -190,130 +203,32 @@ class RawSegment(BaseSegment):
     ) -> "RawSegment":
         """Create a new segment, with exactly the same position but different content.
 
+        Args:
+            raw (Optional[str]): The new content for the segment.
+            source_fixes (Optional[List[SourceFix]]): A list of fixes to be applied to
+                the segment.
+
         Returns:
-            A copy of this object with new contents.
+            RawSegment: A copy of this object with new contents.
 
         Used mostly by fixes.
 
-        NOTE: This *doesn't* copy the uuid. The edited segment is a new segment.
+        NOTE: This *doesn't* copy the uuid. The edited segment is a new
+        segment.
 
         """
         return self.__class__(
             raw=raw or self.raw,
             pos_marker=self.pos_marker,
-            type=self._surrogate_type,
+            instance_types=self.instance_types,
             trim_start=self.trim_start,
             trim_chars=self.trim_chars,
             source_fixes=source_fixes or self.source_fixes,
         )
 
 
-class CodeSegment(RawSegment):
-    """An alias for RawSegment.
-
-    This has a more explicit name for segment creation.
-    """
-
-    pass
-
-
-class UnlexableSegment(CodeSegment):
-    """A placeholder to unlexable sections.
-
-    This otherwise behaves exactly like a code section.
-    """
-
-    type = "unlexable"
-
-
-class CommentSegment(RawSegment):
-    """Segment containing a comment."""
-
-    type = "comment"
-    _is_code = False
-    _is_comment = True
-
-
-class WhitespaceSegment(RawSegment):
-    """Segment containing whitespace."""
-
-    type = "whitespace"
-    _is_whitespace = True
-    _is_code = False
-    _is_comment = False
-    _default_raw = " "
-
-
-class NewlineSegment(RawSegment):
-    """Segment containing a newline.
-
-    NOTE: NewlineSegment does not inherit from WhitespaceSegment.
-    Therefore NewlineSegment.is_type('whitespace') returns False.
-
-    This is intentional and convenient for rules. If users want
-    to match on both, call .is_type('whitespace', 'newline')
-    """
-
-    type = "newline"
-    _is_whitespace = True
-    _is_code = False
-    _is_comment = False
-    _default_raw = "\n"
-
-
-class KeywordSegment(CodeSegment):
-    """A segment used for matching single words.
-
-    We rename the segment class here so that descendants of
-    _ProtoKeywordSegment can use the same functionality
-    but don't end up being labelled as a `keyword` later.
-    """
-
-    type = "keyword"
-
-    def __init__(
-        self,
-        raw: Optional[str] = None,
-        pos_marker: Optional[PositionMarker] = None,
-        type: Optional[str] = None,
-        source_fixes: Optional[List[SourceFix]] = None,
-        trim_chars: Optional[Tuple[str, ...]] = None,
-    ):
-        """If no other name is provided we extrapolate it from the raw."""
-        super().__init__(
-            raw=raw,
-            pos_marker=pos_marker,
-            type=type,
-            source_fixes=source_fixes,
-        )
-
-    def edit(
-        self, raw: Optional[str] = None, source_fixes: Optional[List[SourceFix]] = None
-    ) -> "KeywordSegment":
-        """Create a new segment, with exactly the same position but different content.
-
-        Returns:
-            A copy of this object with new contents.
-
-        Used mostly by fixes.
-
-        NOTE: This *doesn't* copy the uuid. The edited segment is a new segment.
-
-        """
-        return self.__class__(
-            raw=raw or self.raw,
-            pos_marker=self.pos_marker,
-            type=self._surrogate_type,
-            source_fixes=source_fixes or self.source_fixes,
-        )
-
-
-class SymbolSegment(CodeSegment):
-    """A segment used for matching single entities which aren't keywords.
-
-    We rename the segment class here so that descendants of
-    _ProtoKeywordSegment can use the same functionality
-    but don't end up being labelled as a `keyword` later.
-    """
-
-    type = "symbol"
+__all__ = [
+    "PositionMarker",
+    "RawSegment",
+    "SourceFix",
+]
