@@ -7,9 +7,10 @@ This mapping is used to validate rule config inputs, as well
 as document rule configuration.
 """
 
-from sqlfluff.core.plugin.host import get_plugin_manager
 from typing import Any, Dict
 
+from sqlfluff.core.plugin.host import get_plugin_manager
+
 STANDARD_CONFIG_INFO_DICT: Dict[str, Dict[str, Any]] = {
     "tab_space_size": {
         "validation": range(100),
@@ -59,9 +60,10 @@ STANDARD_CONFIG_INFO_DICT: Dict[str, Dict[str, Any]] = {
         "definition": "The capitalisation policy to enforce.",
     },
     "extended_capitalisation_policy": {
-        "validation": ["consistent", "upper", "lower", "pascal", "capitalise"],
+        "validation": ["consistent", "upper", "lower", "pascal", "capitalise", "snake"],
         "definition": (
-            "The capitalisation policy to enforce, extended with PascalCase. "
+            "The capitalisation policy to enforce, extended with PascalCase "
+            "and snake_case. "
             "This is separate from ``capitalisation_policy`` as it should not be "
             "applied to keywords."
         ),
@@ -213,11 +215,24 @@ STANDARD_CONFIG_INFO_DICT: Dict[str, Dict[str, Any]] = {
         "validation": ["consistent", "shorthand", "convert", "cast"],
         "definition": ("The expectation for using sql type casting"),
     },
+    "preferred_first_table_in_join_clause": {
+        "validation": ["earlier", "later"],
+        "definition": (
+            "Which table to list first when joining two tables. "
+            "Defaults to ``earlier``."
+        ),
+    },
+    "preferred_not_equal_style": {
+        "validation": ["consistent", "c_style", "ansi"],
+        "definition": (
+            "The style for using not equal to operator. Defaults to ``consistent``."
+        ),
+    },
 }
 
 
 def get_config_info() -> Dict[str, Any]:
-    """Gets the config from core sqlfluff and sqlfluff plugins and merges them."""
+    """Get the config from core sqlfluff and sqlfluff plugins and merges them."""
     plugin_manager = get_plugin_manager()
     configs_info = plugin_manager.hook.get_configs_info()
     return {
