@@ -17,13 +17,15 @@ from sqlfluff.core.parser import (
     CodeSegment,
     Dedent,
     Delimited,
-    GreedyUntil,
+    IdentifierSegment,
     Indent,
+    LiteralSegment,
     Matchable,
-    TypedParser,
+    MultiStringParser,
     Nothing,
     OneOf,
     OptionallyBracketed,
+    ParseMode,
     Ref,
     RegexLexer,
     RegexParser,
@@ -32,13 +34,13 @@ from sqlfluff.core.parser import (
     StringLexer,
     StringParser,
     SymbolSegment,
-    MultiStringParser,
+    TypedParser,
 )
+from sqlfluff.dialects import dialect_ansi as ansi
 from sqlfluff.dialects.dialect_bigquery_keywords import (
     bigquery_reserved_keywords,
     bigquery_unreserved_keywords,
 )
-from sqlfluff.dialects import dialect_ansi as ansi
 
 ansi_dialect = load_raw_dialect("ansi")
 bigquery_dialect = ansi_dialect.copy_as("bigquery")
@@ -51,8 +53,14 @@ bigquery_dialect.insert_lexer_matchers(
         RegexLexer(
             "at_sign_literal",
             r"@[a-zA-Z_][\w]*",
-            ansi.LiteralSegment,
-            segment_kwargs={"type": "at_sign_literal", "trim_chars": ("@",)},
+            LiteralSegment,
+            segment_kwargs={"trim_chars": ("@",)},
+        ),
+        RegexLexer(
+            "double_at_sign_literal",
+            r"@@[a-zA-Z_][\w]*",
+            LiteralSegment,
+            segment_kwargs={"trim_chars": ("@@",)},
         ),
     ],
     before="equals",
@@ -71,7 +79,6 @@ bigquery_dialect.patch_lexer_matchers(
             r"([rR]?[bB]?|[bB]?[rR]?)?('''((?<!\\)(\\{2})*\\'|'{,2}(?!')|[^'])"
             r"*(?<!\\)(\\{2})*'''|'((?<!\\)(\\{2})*\\'|[^'])*(?<!\\)(\\{2})*')",
             CodeSegment,
-            segment_kwargs={"type": "single_quote"},
         ),
         RegexLexer(
             "double_quote",
@@ -79,7 +86,6 @@ bigquery_dialect.patch_lexer_matchers(
             r'|[^\"])*(?<!\\)(\\{2})*\"\"\"|"((?<!\\)(\\{2})*\\"|[^"])*(?<!\\)'
             r'(\\{2})*")',
             CodeSegment,
-            segment_kwargs={"type": "double_quote"},
         ),
     ]
 )
@@ -87,13 +93,13 @@ bigquery_dialect.patch_lexer_matchers(
 bigquery_dialect.add(
     DoubleQuotedLiteralSegment=TypedParser(
         "double_quote",
-        ansi.LiteralSegment,
+        LiteralSegment,
         type="quoted_literal",
         trim_chars=('"',),
     ),
     SingleQuotedLiteralSegment=TypedParser(
         "single_quote",
-        ansi.LiteralSegment,
+        LiteralSegment,
         type="quoted_literal",
         trim_chars=("'",),
     ),
@@ -122,12 +128,18 @@ bigquery_dialect.add(
     QuestionMarkSegment=StringParser("?", SymbolSegment, type="question_mark"),
     AtSignLiteralSegment=TypedParser(
         "at_sign_literal",
-        ansi.LiteralSegment,
+        LiteralSegment,
+        type="at_sign_literal",
+    ),
+    DoubleAtSignLiteralSegment=TypedParser(
+        "double_at_sign_literal",
+        LiteralSegment,
+        type="double_at_sign_literal",
     ),
     # Add a Full equivalent which also allow keywords
     NakedIdentifierFullSegment=RegexParser(
         r"[A-Z_][A-Z0-9_]*",
-        ansi.IdentifierSegment,
+        IdentifierSegment,
         type="naked_identifier_all",
     ),
     NakedIdentifierPart=RegexParser(
@@ -135,7 +147,7 @@ bigquery_dialect.add(
         # NOTE: This one can match an "all numbers" variant.
         # https://cloud.google.com/resource-manager/docs/creating-managing-projects
         r"[A-Z0-9_]+",
-        ansi.IdentifierSegment,
+        IdentifierSegment,
         type="naked_identifier",
     ),
     SingleIdentifierFullGrammar=OneOf(
@@ -198,7 +210,7 @@ bigquery_dialect.replace(
         # Generate the anti template from the set of reserved keywords
         lambda dialect: RegexParser(
             r"[A-Z_][A-Z0-9_]*",
-            ansi.IdentifierSegment,
+            IdentifierSegment,
             type="naked_identifier",
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
         )
@@ -210,6 +222,7 @@ bigquery_dialect.replace(
             Ref("ExpressionSegment"),
             Sequence(OneOf("IGNORE", "RESPECT"), "NULLS", optional=True),
         ),
+        Sequence(Ref("ExpressionSegment"), "HAVING", OneOf("MIN", "MAX")),
         Ref("NamedArgumentSegment"),
     ),
     TrimParametersGrammar=Nothing(),
@@ -221,9 +234,7 @@ bigquery_dialect.replace(
     ),
     DateTimeLiteralGrammar=Sequence(
         OneOf("DATE", "DATETIME", "TIME", "TIMESTAMP"),
-        TypedParser(
-            "single_quote", ansi.LiteralSegment, type="date_constructor_literal"
-        ),
+        TypedParser("single_quote", LiteralSegment, type="date_constructor_literal"),
     ),
     JoinLikeClauseGrammar=Sequence(
         AnyNumberOf(
@@ -233,15 +244,16 @@ bigquery_dialect.replace(
         ),
         Ref("AliasExpressionSegment", optional=True),
     ),
+    ConditionalCrossJoinKeywordsGrammar=Nothing(),
     NaturalJoinKeywordsGrammar=Nothing(),
+    UnconditionalCrossJoinKeywordsGrammar=Ref.keyword("CROSS"),
     MergeIntoLiteralGrammar=Sequence("MERGE", Ref.keyword("INTO", optional=True)),
-    Accessor_Grammar=AnyNumberOf(
+    AccessorGrammar=AnyNumberOf(
         Ref("ArrayAccessorSegment"),
         # Add in semi structured expressions
         Ref("SemiStructuredAccessorSegment"),
     ),
-    PrimaryKeyGrammar=Nothing(),
-    ForeignKeyGrammar=Nothing(),
+    BracketedSetExpressionGrammar=Bracketed(Ref("SetExpressionSegment")),
 )
 
 
@@ -353,35 +365,10 @@ class SetOperatorSegment(BaseSegment):
     )
 
 
-class SetExpressionSegment(ansi.SetExpressionSegment):
-    """A set expression with either Union, Minus, Except or Intersect."""
-
-    match_grammar: Matchable = Sequence(
-        OneOf(
-            Ref("NonSetSelectableGrammar"),
-            Bracketed(Ref("SetExpressionSegment")),
-        ),
-        AnyNumberOf(
-            Sequence(
-                Ref("SetOperatorSegment"),
-                OneOf(
-                    Ref("NonSetSelectableGrammar"),
-                    Bracketed(Ref("SetExpressionSegment")),
-                ),
-            ),
-            min_times=1,
-        ),
-        Ref("OrderByClauseSegment", optional=True),
-        Ref("LimitClauseSegment", optional=True),
-        Ref("NamedWindowSegment", optional=True),
-    )
-
-
 class SelectStatementSegment(ansi.SelectStatementSegment):
     """Enhance `SELECT` statement to include QUALIFY."""
 
-    match_grammar = ansi.SelectStatementSegment.match_grammar
-    parse_grammar = ansi.SelectStatementSegment.parse_grammar.copy(
+    match_grammar = ansi.SelectStatementSegment.match_grammar.copy(
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OrderByClauseSegment", optional=True),
     )
@@ -390,8 +377,7 @@ class SelectStatementSegment(ansi.SelectStatementSegment):
 class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
     """Enhance unordered `SELECT` statement to include QUALIFY."""
 
-    match_grammar = ansi.UnorderedSelectStatementSegment.match_grammar
-    parse_grammar = ansi.UnorderedSelectStatementSegment.parse_grammar.copy(
+    match_grammar = ansi.UnorderedSelectStatementSegment.match_grammar.copy(
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OverlapsClauseSegment", optional=True),
     )
@@ -408,6 +394,7 @@ class MultiStatementSegment(BaseSegment):
         Ref("LoopStatementSegment"),
         Ref("IfStatementSegment"),
         Ref("CreateProcedureStatementSegment"),
+        Ref("BeginStatementSegment"),
     )
 
 
@@ -421,7 +408,7 @@ class FileSegment(BaseFileSegment):
 
     # NB: We don't need a match_grammar here because we're
     # going straight into instantiating it directly usually.
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         Sequence(
             OneOf(
                 Ref("MultiStatementSegment"),
@@ -442,8 +429,7 @@ class FileSegment(BaseFileSegment):
 class StatementSegment(ansi.StatementSegment):
     """Overriding StatementSegment to allow for additional segment parsing."""
 
-    match_grammar = ansi.StatementSegment.match_grammar
-    parse_grammar = ansi.StatementSegment.parse_grammar.copy(
+    match_grammar = ansi.StatementSegment.match_grammar.copy(
         insert=[
             Ref("DeclareStatementSegment"),
             Ref("SetStatementSegment"),
@@ -489,8 +475,7 @@ class ForInStatementsSegment(BaseSegment):
     """
 
     type = "for_in_statements"
-    match_grammar = GreedyUntil(Sequence("END", "FOR"))
-    parse_grammar = AnyNumberOf(
+    match_grammar = AnyNumberOf(
         Sequence(
             OneOf(
                 Ref("StatementSegment"),
@@ -498,6 +483,8 @@ class ForInStatementsSegment(BaseSegment):
             ),
             Ref("DelimiterGrammar"),
         ),
+        terminators=[Sequence("END", "FOR")],
+        parse_mode=ParseMode.GREEDY,
     )
 
 
@@ -531,8 +518,7 @@ class RepeatStatementsSegment(BaseSegment):
     """
 
     type = "repeat_statements"
-    match_grammar = GreedyUntil(Ref.keyword("UNTIL"))
-    parse_grammar = AnyNumberOf(
+    match_grammar = AnyNumberOf(
         Sequence(
             OneOf(
                 Ref("StatementSegment"),
@@ -540,6 +526,8 @@ class RepeatStatementsSegment(BaseSegment):
             ),
             Ref("DelimiterGrammar"),
         ),
+        terminators=["UNTIL"],
+        parse_mode=ParseMode.GREEDY,
     )
 
 
@@ -569,8 +557,7 @@ class IfStatementsSegment(BaseSegment):
     """
 
     type = "if_statements"
-    match_grammar = GreedyUntil(OneOf("ELSE", "ELSEIF", Sequence("END", "IF")))
-    parse_grammar = AnyNumberOf(
+    match_grammar = AnyNumberOf(
         Sequence(
             OneOf(
                 Ref("StatementSegment"),
@@ -578,6 +565,12 @@ class IfStatementsSegment(BaseSegment):
             ),
             Ref("DelimiterGrammar"),
         ),
+        terminators=[
+            "ELSE",
+            "ELSEIF",
+            Sequence("END", "IF"),
+        ],
+        parse_mode=ParseMode.GREEDY,
     )
 
 
@@ -624,8 +617,7 @@ class LoopStatementsSegment(BaseSegment):
     """
 
     type = "loop_statements"
-    match_grammar = GreedyUntil(Sequence("END", "LOOP"))
-    parse_grammar = AnyNumberOf(
+    match_grammar = AnyNumberOf(
         Sequence(
             OneOf(
                 Ref("StatementSegment"),
@@ -633,6 +625,8 @@ class LoopStatementsSegment(BaseSegment):
             ),
             Ref("DelimiterGrammar"),
         ),
+        terminators=[Sequence("END", "LOOP")],
+        parse_mode=ParseMode.GREEDY,
     )
 
 
@@ -660,12 +654,13 @@ class WhileStatementsSegment(BaseSegment):
     """
 
     type = "while_statements"
-    match_grammar = GreedyUntil(Sequence("END", "WHILE"))
-    parse_grammar = AnyNumberOf(
+    match_grammar = AnyNumberOf(
         Sequence(
             Ref("StatementSegment"),
             Ref("DelimiterGrammar"),
         ),
+        terminators=[Sequence("END", "WHILE")],
+        parse_mode=ParseMode.GREEDY,
     )
 
 
@@ -720,13 +715,13 @@ class IntervalExpressionSegment(ansi.IntervalExpressionSegment):
 bigquery_dialect.replace(
     QuotedIdentifierSegment=TypedParser(
         "back_quote",
-        ansi.IdentifierSegment,
+        IdentifierSegment,
         type="quoted_identifier",
         trim_chars=("`",),
     ),
     # Add ParameterizedSegment to the ansi NumericLiteralSegment
     NumericLiteralSegment=OneOf(
-        TypedParser("numeric_literal", ansi.LiteralSegment, type="numeric_literal"),
+        TypedParser("numeric_literal", LiteralSegment, type="numeric_literal"),
         Ref("ParameterizedSegment"),
     ),
     QuotedLiteralSegment=OneOf(
@@ -737,11 +732,17 @@ bigquery_dialect.replace(
     LiteralGrammar=ansi_dialect.get_grammar("LiteralGrammar").copy(
         insert=[
             Ref("ParameterizedSegment"),
+            Ref("SystemVariableSegment"),
         ]
     ),
     PostTableExpressionGrammar=Sequence(
         Sequence(
-            "FOR", "SYSTEM_TIME", "AS", "OF", Ref("ExpressionSegment"), optional=True
+            "FOR",
+            OneOf("SYSTEM_TIME", Sequence("SYSTEM", "TIME")),
+            "AS",
+            "OF",
+            Ref("ExpressionSegment"),
+            optional=True,
         ),
         Sequence(
             "WITH",
@@ -870,7 +871,7 @@ class FunctionNameSegment(ansi.FunctionNameSegment):
             Ref("QuotedIdentifierSegment"),
             terminators=[Ref("BracketedSegment")],
         ),
-        # BigQuery allows whitespaces between the `.` of a function refrence or
+        # BigQuery allows whitespaces between the `.` of a function reference or
         # SAFE prefix. Keeping the explicit `allow_gaps=True` here to
         # make the distinction from `ansi.FunctionNameSegment` clear.
         allow_gaps=True,
@@ -928,9 +929,9 @@ class FunctionSegment(ansi.FunctionSegment):
                         Ref("DatePartWeekSegment"),
                         Ref(
                             "FunctionContentsGrammar",
-                            ephemeral_name="FunctionContentsGrammar",
                         ),
                     ),
+                    parse_mode=ParseMode.GREEDY,
                 ),
             ),
             Sequence(
@@ -948,8 +949,8 @@ class FunctionSegment(ansi.FunctionSegment):
                             "FunctionContentsGrammar",
                             # The brackets might be empty for some functions...
                             optional=True,
-                            ephemeral_name="FunctionContentsGrammar",
-                        )
+                        ),
+                        parse_mode=ParseMode.GREEDY,
                     ),
                 ),
                 # Functions returning ARRAYS in BigQuery can have optional
@@ -1030,6 +1031,49 @@ class ExceptClauseSegment(BaseSegment):
     )
 
 
+class BeginStatementSegment(BaseSegment):
+    """A `BEGIN...EXCEPTION...END` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/procedural-language#beginexceptionend
+    """
+
+    type = "begin_statement"
+
+    match_grammar = Sequence(
+        "BEGIN",
+        Indent,
+        AnyNumberOf(
+            Sequence(
+                Ref("StatementSegment"),
+                Ref("DelimiterGrammar"),
+            ),
+            min_times=1,
+            terminators=["END", "EXCEPTION"],
+            parse_mode=ParseMode.GREEDY,
+        ),
+        Dedent,
+        Sequence(
+            "EXCEPTION",
+            "WHEN",
+            "ERROR",
+            "THEN",
+            Indent,
+            AnyNumberOf(
+                Sequence(
+                    Ref("StatementSegment"),
+                    Ref("DelimiterGrammar"),
+                ),
+                min_times=1,
+                terminators=["END"],
+                parse_mode=ParseMode.GREEDY,
+            ),
+            Dedent,
+            optional=True,
+        ),
+        "END",
+    )
+
+
 class ReplaceClauseSegment(BaseSegment):
     """SELECT REPLACE clause."""
 
@@ -1138,14 +1182,6 @@ class NamedArgumentSegment(BaseSegment):
     )
 
 
-# Inherit from the ANSI ObjectReferenceSegment this way so we can inherit
-# other segment types from it.
-class ObjectReferenceSegment(ansi.ObjectReferenceSegment):
-    """A reference to an object."""
-
-    pass
-
-
 class SemiStructuredAccessorSegment(BaseSegment):
     """A semi-structured data accessor segment."""
 
@@ -1168,7 +1204,7 @@ class SemiStructuredAccessorSegment(BaseSegment):
     )
 
 
-class ColumnReferenceSegment(ObjectReferenceSegment):
+class ColumnReferenceSegment(ansi.ObjectReferenceSegment):
     """A reference to column, field or alias.
 
     We override this for BigQuery to allow keywords in structures
@@ -1184,12 +1220,10 @@ class ColumnReferenceSegment(ObjectReferenceSegment):
     match_grammar: Matchable = Sequence(
         Ref("SingleIdentifierGrammar"),
         Sequence(
-            OneOf(Ref("DotSegment"), Sequence(Ref("DotSegment"), Ref("DotSegment"))),
+            Ref("ObjectReferenceDelimiterGrammar"),
             Delimited(
                 Ref("SingleIdentifierFullGrammar"),
-                delimiter=OneOf(
-                    Ref("DotSegment"), Sequence(Ref("DotSegment"), Ref("DotSegment"))
-                ),
+                delimiter=Ref("ObjectReferenceDelimiterGrammar"),
                 terminators=[
                     "ON",
                     "AS",
@@ -1257,7 +1291,7 @@ class ColumnReferenceSegment(ObjectReferenceSegment):
         return super().extract_possible_multipart_references(levels)
 
 
-class TableReferenceSegment(ObjectReferenceSegment):
+class TableReferenceSegment(ansi.ObjectReferenceSegment):
     """A reference to an object that may contain embedded hyphens."""
 
     type = "table_reference"
@@ -1275,9 +1309,7 @@ class TableReferenceSegment(ObjectReferenceSegment):
             ),
             allow_gaps=False,
         ),
-        delimiter=OneOf(
-            Ref("DotSegment"), Sequence(Ref("DotSegment"), Ref("DotSegment"))
-        ),
+        delimiter=Ref("ObjectReferenceDelimiterGrammar"),
         terminators=[
             "ON",
             "AS",
@@ -1342,6 +1374,19 @@ class TableReferenceSegment(ObjectReferenceSegment):
             yield flush()
 
 
+class SystemVariableSegment(BaseSegment):
+    """BigQuery supports usage of system-level variables, which are prefixed with @@.
+
+    These are also used in exception blocks in the @@error object.
+
+    https://cloud.google.com/bigquery/docs/reference/system-variables
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/procedural-language#beginexceptionend
+    """
+
+    type = "system_variable"
+    match_grammar = Ref("DoubleAtSignLiteralSegment")
+
+
 class DeclareStatementSegment(BaseSegment):
     """Declaration of a variable.
 
@@ -1440,6 +1485,32 @@ class OptionsSegment(BaseSegment):
     )
 
 
+class TableConstraintSegment(ansi.TableConstraintSegment):
+    """A table constraint segment.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#table_constraints
+    """
+
+    type = "table_constraint"
+    match_grammar = OneOf(
+        Sequence(
+            Ref("PrimaryKeyGrammar"),
+            Ref("BracketedColumnReferenceListGrammar"),
+            "NOT",
+            "ENFORCED",
+        ),
+        Sequence(
+            Ref("ForeignKeyGrammar"),
+            Ref("BracketedColumnReferenceListGrammar"),
+            "REFERENCES",
+            Ref("TableReferenceSegment"),
+            Ref("BracketedColumnReferenceListGrammar"),
+            "NOT",
+            "ENFORCED",
+        ),
+    )
+
+
 class ColumnDefinitionSegment(ansi.ColumnDefinitionSegment):
     """A column definition, e.g. for CREATE TABLE or ALTER TABLE.
 
@@ -1454,10 +1525,21 @@ class ColumnDefinitionSegment(ansi.ColumnDefinitionSegment):
     )
 
 
+class ViewColumnDefinitionSegment(ansi.ColumnDefinitionSegment):
+    """A column definition, for view_column_name_list of CREATE VIEW statement."""
+
+    match_grammar: Matchable = Sequence(
+        Ref("SingleIdentifierGrammar"),  # Column name
+        Ref("OptionsSegment", optional=True),
+    )
+
+
 class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
-    """A `CREATE TABLE` statement."""
+    """`CREATE TABLE` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_table_statement
+    """
 
-    # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_table_statement
     match_grammar = Sequence(
         "CREATE",
         Ref("OrReplaceGrammar", optional=True),
@@ -1474,7 +1556,10 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
         Sequence(
             Bracketed(
                 Delimited(
-                    Ref("ColumnDefinitionSegment"),
+                    OneOf(
+                        Ref("ColumnDefinitionSegment"),
+                        Ref("TableConstraintSegment"),
+                    ),
                     allow_trailing=True,
                 )
             ),
@@ -1587,10 +1672,10 @@ class CreateExternalTableStatementSegment(BaseSegment):
 
     match_grammar = Sequence(
         "CREATE",
-        Sequence("OR", "REPLACE", optional=True),
+        Ref("OrReplaceGrammar", optional=True),
         "EXTERNAL",
         "TABLE",
-        Sequence("IF", "NOT", "EXISTS", optional=True),
+        Ref("IfNotExistsGrammar", optional=True),
         Ref("TableReferenceSegment"),
         Bracketed(
             Delimited(
@@ -1635,7 +1720,12 @@ class CreateViewStatementSegment(ansi.CreateViewStatementSegment):
         Ref("IfNotExistsGrammar", optional=True),
         Ref("TableReferenceSegment"),
         # Optional list of column names
-        Ref("BracketedColumnReferenceListGrammar", optional=True),
+        Bracketed(
+            Delimited(
+                Ref("ViewColumnDefinitionSegment"),
+            ),
+            optional=True,
+        ),
         Ref("OptionsSegment", optional=True),
         "AS",
         OptionallyBracketed(Ref("SelectableGrammar")),
@@ -1736,8 +1826,11 @@ class PivotForClauseSegment(BaseSegment):
     """
 
     type = "pivot_for_clause"
-    match_grammar = GreedyUntil("IN")
-    parse_grammar = Ref("BaseExpressionElementGrammar")
+    match_grammar = Sequence(
+        Ref("BaseExpressionElementGrammar"),
+        terminators=["IN"],
+        parse_mode=ParseMode.GREEDY,
+    )
 
 
 class FromPivotExpressionSegment(BaseSegment):
@@ -1911,7 +2004,7 @@ class MergeNotMatchedBySourceClauseSegment(ansi.MergeMatchedClauseSegment):
 
     It inherits from `ansi.MergeMatchedClauseSegment` because NotMatchedBySource clause
     is conceptionally more close to a Matched clause than to NotMatched clause, i.e.
-    it get's combined with an UPDATE or DELETE, not with an INSERT.
+    it gets combined with an UPDATE or DELETE, not with an INSERT.
     """
 
     type = "merge_when_matched_clause"
@@ -2045,7 +2138,7 @@ class ExportStatementSegment(BaseSegment):
 
 
 class ProcedureNameSegment(BaseSegment):
-    """Prcoedure name, including any prefix bits, e.g. project or schema."""
+    """Procedure name, including any prefix bits, e.g. project or schema."""
 
     type = "procedure_name"
     match_grammar: Matchable = Sequence(
@@ -2085,12 +2178,13 @@ class ProcedureStatements(BaseSegment):
     """
 
     type = "procedure_statements"
-    match_grammar = GreedyUntil("END")
-    parse_grammar = AnyNumberOf(
+    match_grammar = AnyNumberOf(
         Sequence(
             Ref("StatementSegment"),
             Ref("DelimiterGrammar"),
         ),
+        terminators=["END"],
+        parse_mode=ParseMode.GREEDY,
     )
 
 
