@@ -2,6 +2,7 @@
 
 https://clickhouse.com/
 """
+
 from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
     AnyNumberOf,
@@ -11,119 +12,40 @@ from sqlfluff.core.parser import (
     Conditional,
     Dedent,
     Delimited,
+    IdentifierSegment,
     Indent,
+    LiteralSegment,
     Matchable,
+    Nothing,
     OneOf,
     OptionallyBracketed,
+    ParseMode,
     Ref,
     Sequence,
+    StringLexer,
     SymbolSegment,
     TypedParser,
-    StringLexer,
 )
 from sqlfluff.dialects import dialect_ansi as ansi
-from sqlfluff.dialects.dialect_clickhouse_keywords import (
-    UNRESERVED_KEYWORDS,
-)
+from sqlfluff.dialects.dialect_clickhouse_keywords import UNRESERVED_KEYWORDS
 
 ansi_dialect = load_raw_dialect("ansi")
 
 clickhouse_dialect = ansi_dialect.copy_as("clickhouse")
 clickhouse_dialect.sets("unreserved_keywords").update(UNRESERVED_KEYWORDS)
-clickhouse_dialect.replace(
-    SingleIdentifierGrammar=OneOf(
-        Ref("NakedIdentifierSegment"),
-        Ref("QuotedIdentifierSegment"),
-        Ref("SingleQuotedIdentifierSegment"),
-    ),
-    QuotedLiteralSegment=OneOf(
-        TypedParser(
-            "single_quote",
-            ansi.LiteralSegment,
-            type="quoted_literal",
-        ),
-        TypedParser(
-            "dollar_quote",
-            ansi.LiteralSegment,
-            type="quoted_literal",
-        ),
-    ),
-)
+
 
 clickhouse_dialect.insert_lexer_matchers(
     # https://clickhouse.com/docs/en/sql-reference/functions#higher-order-functions---operator-and-lambdaparams-expr-function
-    [StringLexer("lambda", r"->", SymbolSegment, segment_kwargs={"type": "lambda"})],
+    [StringLexer("lambda", r"->", SymbolSegment)],
     before="newline",
 )
 
 clickhouse_dialect.add(
-    JoinTypeKeywords=OneOf(
-        # This case INNER [ANY,ALL] JOIN
-        Sequence("INNER", OneOf("ALL", "ANY", optional=True)),
-        # This case [ANY,ALL] INNER JOIN
-        Sequence(OneOf("ALL", "ANY", optional=True), "INNER"),
-        # This case FULL ALL OUTER JOIN
-        Sequence(
-            "FULL",
-            Ref.keyword("ALL", optional=True),
-            Ref.keyword("OUTER", optional=True),
-        ),
-        # This case ALL FULL OUTER JOIN
-        Sequence(
-            Ref.keyword("ALL", optional=True),
-            "FULL",
-            Ref.keyword("OUTER", optional=True),
-        ),
-        # This case LEFT [OUTER,ANTI,SEMI,ANY,ASOF] JOIN
-        Sequence(
-            "LEFT",
-            OneOf(
-                "ANTI",
-                "SEMI",
-                OneOf("ANY", "ALL", optional=True),
-                "ASOF",
-                optional=True,
-            ),
-            Ref.keyword("OUTER", optional=True),
-        ),
-        # This case [ANTI,SEMI,ANY,ASOF] LEFT JOIN
-        Sequence(
-            OneOf(
-                "ANTI",
-                "SEMI",
-                OneOf("ANY", "ALL", optional=True),
-                "ASOF",
-            ),
-            "LEFT",
-        ),
-        # This case RIGHT [OUTER,ANTI,SEMI,ANY,ASOF] JOIN
-        Sequence(
-            "RIGHT",
-            OneOf(
-                "OUTER",
-                "ANTI",
-                "SEMI",
-                OneOf("ANY", "ALL", optional=True),
-                optional=True,
-            ),
-            Ref.keyword("OUTER", optional=True),
-        ),
-        # This case [OUTER,ANTI,SEMI,ANY] RIGHT JOIN
-        Sequence(
-            OneOf(
-                "ANTI",
-                "SEMI",
-                OneOf("ANY", "ALL", optional=True),
-                optional=True,
-            ),
-            "RIGHT",
-        ),
-        # This case CROSS JOIN
-        "CROSS",
-        # This case ANY JOIN
-        "ANY",
-        # This case ALL JOIN
-        "ALL",
+    BackQuotedIdentifierSegment=TypedParser(
+        "back_quote",
+        IdentifierSegment,
+        type="quoted_identifier",
     ),
     LambdaFunctionSegment=TypedParser("lambda", SymbolSegment, type="lambda"),
 )
@@ -137,9 +59,102 @@ clickhouse_dialect.replace(
         # Add Lambda Function
         Ref("LambdaFunctionSegment"),
     ),
-)
-
-clickhouse_dialect.replace(
+    # https://clickhouse.com/docs/en/sql-reference/statements/select/join/#supported-types-of-join
+    JoinTypeKeywordsGrammar=Sequence(
+        Ref.keyword("GLOBAL", optional=True),
+        OneOf(
+            # This case INNER [ANY,ALL] JOIN
+            Sequence("INNER", OneOf("ALL", "ANY", optional=True)),
+            # This case [ANY,ALL] INNER JOIN
+            Sequence(OneOf("ALL", "ANY", optional=True), "INNER"),
+            # This case FULL ALL OUTER JOIN
+            Sequence(
+                "FULL",
+                Ref.keyword("ALL", optional=True),
+                Ref.keyword("OUTER", optional=True),
+            ),
+            # This case ALL FULL OUTER JOIN
+            Sequence(
+                Ref.keyword("ALL", optional=True),
+                "FULL",
+                Ref.keyword("OUTER", optional=True),
+            ),
+            # This case LEFT [OUTER,ANTI,SEMI,ANY,ASOF] JOIN
+            Sequence(
+                "LEFT",
+                OneOf(
+                    "ANTI",
+                    "SEMI",
+                    OneOf("ANY", "ALL", optional=True),
+                    "ASOF",
+                    optional=True,
+                ),
+                Ref.keyword("OUTER", optional=True),
+            ),
+            # This case [ANTI,SEMI,ANY,ASOF] LEFT JOIN
+            Sequence(
+                OneOf(
+                    "ANTI",
+                    "SEMI",
+                    OneOf("ANY", "ALL", optional=True),
+                    "ASOF",
+                ),
+                "LEFT",
+            ),
+            # This case RIGHT [OUTER,ANTI,SEMI,ANY,ASOF] JOIN
+            Sequence(
+                "RIGHT",
+                OneOf(
+                    "OUTER",
+                    "ANTI",
+                    "SEMI",
+                    OneOf("ANY", "ALL", optional=True),
+                    optional=True,
+                ),
+                Ref.keyword("OUTER", optional=True),
+            ),
+            # This case [OUTER,ANTI,SEMI,ANY] RIGHT JOIN
+            Sequence(
+                OneOf(
+                    "ANTI",
+                    "SEMI",
+                    OneOf("ANY", "ALL", optional=True),
+                    optional=True,
+                ),
+                "RIGHT",
+            ),
+            # This case ASOF JOIN
+            "ASOF",
+            # This case ANY JOIN
+            "ANY",
+            # This case ALL JOIN
+            "ALL",
+        ),
+    ),
+    JoinUsingConditionGrammar=Sequence(
+        "USING",
+        Conditional(Indent, indented_using_on=False),
+        Delimited(
+            OneOf(
+                Bracketed(
+                    Delimited(Ref("SingleIdentifierGrammar")),
+                    parse_mode=ParseMode.GREEDY,
+                ),
+                Delimited(Ref("SingleIdentifierGrammar")),
+            ),
+        ),
+        Conditional(Dedent, indented_using_on=False),
+    ),
+    ConditionalCrossJoinKeywordsGrammar=Nothing(),
+    UnconditionalCrossJoinKeywordsGrammar=Sequence(
+        Ref.keyword("GLOBAL", optional=True),
+        Ref.keyword("CROSS"),
+    ),
+    HorizontalJoinKeywordsGrammar=Sequence(
+        Ref.keyword("GLOBAL", optional=True),
+        Ref.keyword("PASTE"),
+    ),
+    NaturalJoinKeywordsGrammar=Nothing(),
     JoinLikeClauseGrammar=Sequence(
         AnyNumberOf(
             Ref("ArrayJoinClauseSegment"),
@@ -147,6 +162,28 @@ clickhouse_dialect.replace(
         ),
         Ref("AliasExpressionSegment", optional=True),
     ),
+    QuotedLiteralSegment=OneOf(
+        TypedParser(
+            "single_quote",
+            LiteralSegment,
+            type="quoted_literal",
+        ),
+        TypedParser(
+            "dollar_quote",
+            LiteralSegment,
+            type="quoted_literal",
+        ),
+    ),
+    SingleIdentifierGrammar=OneOf(
+        Ref("NakedIdentifierSegment"),
+        Ref("QuotedIdentifierSegment"),
+        Ref("SingleQuotedIdentifierSegment"),
+        Ref("BackQuotedIdentifierSegment"),
+    ),
+    InOperatorGrammar=ansi_dialect.get_grammar("InOperatorGrammar").copy(
+        insert=[Ref.keyword("GLOBAL", optional=True)],
+        before=Ref.keyword("NOT", optional=True),
+    ),
 )
 
 
@@ -169,46 +206,6 @@ class BracketedArguments(ansi.BracketedArguments):
     )
 
 
-class JoinClauseSegment(ansi.JoinClauseSegment):
-    """Any number of join clauses, including the `JOIN` keyword.
-
-    https://clickhouse.com/docs/en/sql-reference/statements/select/join/#supported-types-of-join
-    """
-
-    match_grammar = OneOf(
-        Sequence(
-            Ref("JoinTypeKeywords", optional=True),
-            Ref("JoinKeywordsGrammar"),
-            Indent,
-            Ref("FromExpressionElementSegment"),
-            Dedent,
-            Conditional(Indent, indented_using_on=True),
-            OneOf(
-                # ON clause
-                Ref("JoinOnConditionSegment"),
-                # USING clause
-                Sequence(
-                    "USING",
-                    Conditional(Indent, indented_using_on=False),
-                    Delimited(
-                        OneOf(
-                            Bracketed(
-                                Delimited(Ref("SingleIdentifierGrammar")),
-                                ephemeral_name="UsingClauseContents",
-                            ),
-                            Delimited(Ref("SingleIdentifierGrammar")),
-                        ),
-                    ),
-                    Conditional(Dedent, indented_using_on=False),
-                ),
-                # Requires True for CROSS JOIN
-                optional=True,
-            ),
-            Conditional(Dedent, indented_using_on=True),
-        ),
-    )
-
-
 class ArrayJoinClauseSegment(BaseSegment):
     """[LEFT] ARRAY JOIN does not support Join conditions and doesn't work as real JOIN.
 
@@ -244,7 +241,8 @@ class CTEDefinitionSegment(ansi.CTEDefinitionSegment):
             "AS",
             Bracketed(
                 # Ephemeral here to subdivide the query.
-                Ref("SelectableGrammar", ephemeral_name="SelectableGrammar")
+                Ref("SelectableGrammar"),
+                parse_mode=ParseMode.GREEDY,
             ),
         ),
         Sequence(
@@ -279,6 +277,52 @@ class AliasExpressionSegment(ansi.AliasExpressionSegment):
     )
 
 
+class WildcardExpressionSegment(ansi.WildcardExpressionSegment):
+    """An extension of the star expression for Clickhouse."""
+
+    match_grammar = ansi.WildcardExpressionSegment.match_grammar.copy(
+        insert=[
+            Ref("ExceptClauseSegment", optional=True),
+        ]
+    )
+
+
+class ExceptClauseSegment(BaseSegment):
+    """A Clickhouse SELECT EXCEPT clause.
+
+    https://clickhouse.com/docs/en/sql-reference/statements/select#except
+    """
+
+    type = "select_except_clause"
+    match_grammar = Sequence(
+        "EXCEPT",
+        OneOf(
+            Bracketed(Delimited(Ref("SingleIdentifierGrammar"))),
+            Ref("SingleIdentifierGrammar"),
+        ),
+    )
+
+
+class SelectClauseModifierSegment(ansi.SelectClauseModifierSegment):
+    """Things that come after SELECT but before the columns.
+
+    Overridden from ANSI to allow DISTINCT ON ()
+    https://clickhouse.com/docs/en/sql-reference/statements/select/distinct
+    """
+
+    match_grammar = OneOf(
+        Sequence(
+            "DISTINCT",
+            Sequence(
+                "ON",
+                Bracketed(Delimited(Ref("ExpressionSegment"))),
+                optional=True,
+            ),
+        ),
+        "ALL",
+    )
+
+
 class FromExpressionElementSegment(ansi.FromExpressionElementSegment):
     """A table expression.
 
@@ -330,10 +374,10 @@ class TableEngineFunctionSegment(BaseSegment):
                     "FunctionContentsGrammar",
                     # The brackets might be empty for some functions...
                     optional=True,
-                    ephemeral_name="FunctionContentsGrammar",
                 ),
                 # Engine functions may omit brackets.
                 optional=True,
+                parse_mode=ParseMode.GREEDY,
             ),
         ),
     )
@@ -426,10 +470,10 @@ class DatabaseEngineFunctionSegment(BaseSegment):
                     "FunctionContentsGrammar",
                     # The brackets might be empty for some functions...
                     optional=True,
-                    ephemeral_name="FunctionContentsGrammar",
                 ),
                 # Engine functions may omit brackets.
                 optional=True,
+                parse_mode=ParseMode.GREEDY,
             ),
         ),
     )
@@ -1227,8 +1271,7 @@ class SystemStatementSegment(BaseSegment):
 class StatementSegment(ansi.StatementSegment):
     """Overriding StatementSegment to allow for additional segment parsing."""
 
-    match_grammar = ansi.StatementSegment.match_grammar
-    parse_grammar = ansi.StatementSegment.parse_grammar.copy(
+    match_grammar = ansi.StatementSegment.match_grammar.copy(
         insert=[
             Ref("CreateMaterializedViewStatementSegment"),
             Ref("DropDictionaryStatementSegment"),
