@@ -3,18 +3,19 @@
 This is based on postgres dialect, since it was initially based off of Postgres.
 We should monitor in future and see if it should be rebased off of ANSI
 """
+
 from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
     Anything,
     BaseSegment,
     Bracketed,
     Delimited,
+    KeywordSegment,
+    MultiStringParser,
     OneOf,
     Ref,
     Sequence,
-    MultiStringParser,
 )
-from sqlfluff.core.parser.segments.raw import KeywordSegment
 from sqlfluff.dialects import dialect_ansi as ansi
 from sqlfluff.dialects.dialect_materialize_keywords import (
     materialize_reserved_keywords,
@@ -37,13 +38,15 @@ materialize_dialect.update_keywords_set_from_multiline_string(
 class StatementSegment(ansi.StatementSegment):
     """A generic segment, to any of its child subsegments."""
 
-    match_grammar = ansi.StatementSegment.match_grammar
-    parse_grammar = ansi.StatementSegment.parse_grammar.copy(
+    match_grammar = ansi.StatementSegment.match_grammar.copy(
         insert=[
+            Ref("AlterOwnerStatementSegment"),
             Ref("AlterConnectionRotateKeys"),
+            Ref("AlterDefaultPrivilegesStatementSegment"),
             Ref("AlterIndexStatementSegment"),
             Ref("AlterRenameStatementSegment"),
             Ref("AlterSecretStatementSegment"),
+            Ref("AlterSetClusterStatementSegment"),
             Ref("AlterSourceSinkSizeStatementSegment"),
             Ref("CloseStatementSegment"),
             Ref("CopyToStatementSegment"),
@@ -58,10 +61,12 @@ class StatementSegment(ansi.StatementSegment):
             Ref("CreateSourceKafkaStatementSegment"),
             Ref("CreateSourceLoadGeneratorStatementSegment"),
             Ref("CreateSourcePostgresStatementSegment"),
+            Ref("CreateSourceWebhookStatementSegment"),
             Ref("CreateTypeStatementSegment"),
             Ref("CreateViewStatementSegment"),
             Ref("DropStatementSegment"),
             Ref("FetchStatementSegment"),
+            Ref("GrantStatementSegment"),
             Ref("MaterializeExplainStatementSegment"),
             Ref("ShowStatementSegment"),
             Ref("ShowCreateStatementSegment"),
@@ -116,9 +121,46 @@ materialize_dialect.add(
         "CLUSTER",
         Ref("ObjectReferenceSegment"),
     ),
+    Privileges=OneOf(
+        "SELECT",
+        "INSERT",
+        "UPDATE",
+        "DELETE",
+        "CREATE",
+        "USAGE",
+        "CREATEROLE",
+        "CREATEDB",
+        "CREATECLUSTER",
+        Sequence("ALL", Ref.keyword("PRIVILEGES", optional=True)),
+    ),
 )
 
 
+class AlterOwnerStatementSegment(BaseSegment):
+    """A `ALTER OWNER` statement."""
+
+    type = "alter_owner_statement"
+
+    match_grammar = Sequence(
+        "ALTER",
+        OneOf(
+            "CONNECTION",
+            "CLUSTER",
+            Sequence("CLUSTER", "REPLICA"),
+            "INDEX",
+            "SOURCE",
+            "SINK",
+            "VIEW",
+            Sequence("MATERIALIZED", "VIEW"),
+            "TABLE",
+            "SECRET",
+        ),
+        Ref("ObjectReferenceSegment"),
+        Sequence("OWNER", "TO"),
+        Ref("ObjectReferenceSegment"),
+    )
+
+
 class AlterConnectionRotateKeys(BaseSegment):
     """`ALTER CONNECTION` statement."""
 
@@ -134,6 +176,43 @@ class AlterConnectionRotateKeys(BaseSegment):
     )
 
 
+class AlterDefaultPrivilegesStatementSegment(BaseSegment):
+    """A `ALTER DEFAULT PRIVILEGES` statement."""
+
+    type = "alter_default_privileges_statement"
+
+    match_grammar = Sequence(
+        Sequence("ALTER", "DEFAULT", "PRIVILEGES", "FOR"),
+        OneOf(
+            Sequence(
+                OneOf("ROLE", "USER"),
+                Ref("ObjectReferenceSegment"),
+            ),
+            Sequence("ALL", "ROLES"),
+        ),
+        Sequence(
+            "IN",
+            OneOf("SCHEMA", "DATABASE"),
+            Ref("ObjectReferenceSegment"),
+            optional=True,
+        ),
+        "GRANT",
+        Ref("Privileges"),
+        "ON",
+        OneOf(
+            "TABLES",
+            "TYPES",
+            "SECRETS",
+            "CONNECTIONS",
+            "DATABASES",
+            "SCHEMAS",
+            "CLUSTERS",
+        ),
+        "TO",
+        Ref("ObjectReferenceSegment"),
+    )
+
+
 class AlterRenameStatementSegment(BaseSegment):
     """A `ALTER RENAME` statement."""
 
@@ -143,6 +222,7 @@ class AlterRenameStatementSegment(BaseSegment):
         "ALTER",
         OneOf(
             "CONNECTION",
+            Sequence("CLUSTER", Ref.keyword("REPLICA", optional=True)),
             "INDEX",
             "SOURCE",
             "SINK",
@@ -185,6 +265,20 @@ class AlterSecretStatementSegment(BaseSegment):
     )
 
 
+class AlterSetClusterStatementSegment(BaseSegment):
+    """A `ALTER SET CLUSTER` statement."""
+
+    type = "alter_set_cluster_statement"
+
+    match_grammar = Sequence(
+        Sequence("ALTER", "MATERIALIZED", "VIEW"),
+        Ref("IfExistsGrammar", optional=True),
+        Ref("ObjectReferenceSegment"),
+        Sequence("IN", "CLUSTER"),
+        Ref("ObjectReferenceSegment"),
+    )
+
+
 class AlterSourceSinkSizeStatementSegment(BaseSegment):
     """A `ALTER SOURCE/SINK SET SIZE` statement."""
 
@@ -281,14 +375,20 @@ class CreateClusterStatementSegment(BaseSegment):
         "CREATE",
         "CLUSTER",
         Ref("ObjectReferenceSegment"),
-        Sequence(
-            "REPLICAS",
-            Bracketed(
-                Delimited(
-                    Anything(),
-                )
+        OneOf(
+            Sequence(
+                "REPLICAS",
+                Bracketed(
+                    Delimited(
+                        Anything(),
+                    )
+                ),
+                optional=True,
+            ),
+            Sequence(
+                Anything(),
+                optional=True,
             ),
-            optional=True,
         ),
     )
 
@@ -336,11 +436,7 @@ class CreateConnectionStatementSegment(BaseSegment):
                 "TUNNEL",
             ),
         ),
-        Bracketed(
-            Delimited(
-                Anything(),
-            )
-        ),
+        Bracketed(Anything()),
     )
 
 
@@ -448,6 +544,7 @@ class CreateSinkKafkaStatementSegment(BaseSegment):
         "SINK",
         Ref("IfNotExistsGrammar", optional=True),
         Ref("ObjectReferenceSegment"),
+        Ref("InCluster", optional=True),
         "FROM",
         Ref("ObjectReferenceSegment"),
         "INTO",
@@ -495,6 +592,7 @@ class CreateSourceKafkaStatementSegment(BaseSegment):
         "SOURCE",
         Ref("IfNotExistsGrammar", optional=True),
         Ref("ObjectReferenceSegment"),
+        Ref("InCluster", optional=True),
         Bracketed(
             Delimited(
                 Ref("ColumnReferenceSegment"),
@@ -561,6 +659,7 @@ class CreateSourceLoadGeneratorStatementSegment(BaseSegment):
         "SOURCE",
         Ref("IfNotExistsGrammar", optional=True),
         Ref("ObjectReferenceSegment"),
+        Ref("InCluster", optional=True),
         Sequence(
             "FROM",
             "LOAD",
@@ -569,6 +668,7 @@ class CreateSourceLoadGeneratorStatementSegment(BaseSegment):
         OneOf(
             "AUCTION",
             "COUNTER",
+            "MARKETING",
             "TPCH",
         ),
         Bracketed(
@@ -577,21 +677,10 @@ class CreateSourceLoadGeneratorStatementSegment(BaseSegment):
             ),
             optional=True,
         ),
-        OneOf(
-            Sequence(
-                "FOR",
-                "ALL",
-                "TABLES",
-            ),
-            Sequence(
-                "FOR",
-                "TABLES",
-                Bracketed(
-                    Delimited(
-                        Anything(),
-                    )
-                ),
-            ),
+        Sequence(
+            "FOR",
+            "ALL",
+            "TABLES",
             optional=True,
         ),
         Sequence(
@@ -615,6 +704,7 @@ class CreateSourcePostgresStatementSegment(BaseSegment):
         "SOURCE",
         Ref("IfNotExistsGrammar", optional=True),
         Ref("ObjectReferenceSegment"),
+        Ref("InCluster", optional=True),
         Sequence(
             "FROM",
             "POSTGRES",
@@ -656,6 +746,66 @@ class CreateSourcePostgresStatementSegment(BaseSegment):
     )
 
 
+class CreateSourceWebhookStatementSegment(BaseSegment):
+    """A `CREATE SOURCE WEBHOOK` statement."""
+
+    type = "create_source_load_generator_statement"
+    match_grammar = Sequence(
+        "CREATE",
+        "SOURCE",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("ObjectReferenceSegment"),
+        Ref("InCluster", optional=True),
+        Sequence(
+            "FROM",
+            "WEBHOOK",
+            "BODY",
+            "FORMAT",
+        ),
+        OneOf(
+            "TEXT",
+            "JSON",
+            "BYTES",
+        ),
+        OneOf(
+            Sequence(
+                "INCLUDE",
+                "HEADER",
+                Sequence(
+                    Anything(),
+                    optional=True,
+                ),
+            ),
+            Sequence(
+                "INCLUDE",
+                "HEADERS",
+                Bracketed(
+                    Delimited(
+                        Anything(),
+                    )
+                ),
+            ),
+            optional=True,
+        ),
+        Sequence(
+            "CHECK",
+            Bracketed(
+                "WITH",
+                Bracketed(
+                    Delimited(
+                        Anything(),
+                    )
+                ),
+            ),
+            optional=True,
+        ),
+        Sequence(
+            Anything(),
+            optional=True,
+        ),
+    )
+
+
 class CreateTypeStatementSegment(BaseSegment):
     """A `CREATE TYPE` statement."""
 
@@ -973,6 +1123,69 @@ class FetchStatementSegment(BaseSegment):
     )
 
 
+class GrantStatementSegment(BaseSegment):
+    """A `GRANT` statement."""
+
+    type = "grant_statement"
+    match_grammar = Sequence(
+        "GRANT",
+        Ref("Privileges"),
+        "ON",
+        OneOf(
+            Sequence(
+                OneOf(
+                    "TABLE",
+                    "TYPE",
+                    "SECRET",
+                    "CONNECTION",
+                    "DATABASE",
+                    "SCHEMA",
+                    "CLUSTER",
+                    optional=True,
+                ),
+                Delimited(
+                    Ref("ObjectReferenceSegment"),
+                ),
+            ),
+            "SYSTEM",
+            Sequence(
+                "ALL",
+                OneOf(
+                    Sequence(
+                        OneOf(
+                            "TABLES",
+                            "TYPES",
+                            "SECRETS",
+                            "CONNECTIONS",
+                        ),
+                        "IN",
+                        "SCHEMA",
+                        Delimited(
+                            Ref("ObjectReferenceSegment"),
+                        ),
+                    ),
+                    Sequence(
+                        OneOf("TABLES", "TYPES", "SECRETS", "CONNECTIONS", "SCHEMAS"),
+                        "IN",
+                        "DATABASE",
+                        Delimited(
+                            Ref("ObjectReferenceSegment"),
+                        ),
+                    ),
+                    "DATABASES",
+                    "SCHEMAS",
+                    "CLUSTERS",
+                ),
+            ),
+        ),
+        "TO",
+        Sequence("GROUP", optional=True),
+        Delimited(
+            Ref("ObjectReferenceSegment"),
+        ),
+    )
+
+
 class DeclareStatementSegment(BaseSegment):
     """A `DECLARE` statement."""
 
