@@ -10,21 +10,25 @@ from sqlfluff.core.parser import (
     AnySetOf,
     Anything,
     BaseSegment,
+    BinaryOperatorSegment,
     Bracketed,
     CodeSegment,
     CommentSegment,
     Dedent,
     Delimited,
+    IdentifierSegment,
     Indent,
     KeywordSegment,
+    LiteralSegment,
     Matchable,
     OneOf,
     OptionallyBracketed,
+    ParseMode,
     Ref,
     RegexLexer,
     RegexParser,
+    SegmentGenerator,
     Sequence,
-    StartsWith,
     StringLexer,
     StringParser,
     SymbolSegment,
@@ -43,9 +47,9 @@ mysql_dialect.patch_lexer_matchers(
     [
         RegexLexer(
             "inline_comment",
-            r"(-- |#)[^\n]*",
+            r"(^--|-- |#)[^\n]*",
             CommentSegment,
-            segment_kwargs={"trim_start": ("-- ", "#"), "type": "inline_comment"},
+            segment_kwargs={"trim_start": ("--", "#")},
         ),
         # Pattern breakdown:
         # (?s)                     DOTALL (dot matches newline)
@@ -64,13 +68,11 @@ mysql_dialect.patch_lexer_matchers(
             "single_quote",
             r"(?s)('(?:\\'|''|\\\\|[^'])*'(?!'))",
             CodeSegment,
-            segment_kwargs={"type": "single_quote"},
         ),
         RegexLexer(
             "double_quote",
             r'(?s)("(?:\\"|""|\\\\|[^"])*"(?!"))',
             CodeSegment,
-            segment_kwargs={"type": "double_quote"},
         ),
     ]
 )
@@ -80,13 +82,13 @@ mysql_dialect.insert_lexer_matchers(
         RegexLexer(
             "hexadecimal_literal",
             r"([xX]'([\da-fA-F][\da-fA-F])+'|0x[\da-fA-F]+)",
-            ansi.LiteralSegment,
+            LiteralSegment,
             segment_kwargs={"type": "numeric_literal"},
         ),
         RegexLexer(
             "bit_value_literal",
             r"([bB]'[01]+'|0b[01]+)",
-            ansi.LiteralSegment,
+            LiteralSegment,
             segment_kwargs={"type": "numeric_literal"},
         ),
     ],
@@ -139,7 +141,7 @@ mysql_dialect.sets("datetime_units").update(
 mysql_dialect.replace(
     QuotedIdentifierSegment=TypedParser(
         "back_quote",
-        ansi.IdentifierSegment,
+        IdentifierSegment,
         type="quoted_identifier",
         trim_chars=("`",),
     ),
@@ -189,7 +191,7 @@ mysql_dialect.replace(
         OneOf(
             TypedParser(
                 "single_quote",
-                ansi.LiteralSegment,
+                LiteralSegment,
                 type="date_constructor_literal",
             ),
             Ref("NumericLiteralSegment"),
@@ -201,7 +203,7 @@ mysql_dialect.replace(
         # we use grammar to handle this.
         TypedParser(
             "single_quote",
-            ansi.LiteralSegment,
+            LiteralSegment,
             type="quoted_literal",
         ),
         Ref("DoubleQuotedLiteralSegment"),
@@ -222,13 +224,13 @@ mysql_dialect.replace(
         insert=[Ref("SessionVariableNameSegment")]
     ),
     AndOperatorGrammar=OneOf(
-        StringParser("AND", ansi.BinaryOperatorSegment),
-        StringParser("&&", ansi.BinaryOperatorSegment),
+        StringParser("AND", BinaryOperatorSegment),
+        StringParser("&&", BinaryOperatorSegment),
     ),
     OrOperatorGrammar=OneOf(
-        StringParser("OR", ansi.BinaryOperatorSegment),
-        StringParser("||", ansi.BinaryOperatorSegment),
-        StringParser("XOR", ansi.BinaryOperatorSegment),
+        StringParser("OR", BinaryOperatorSegment),
+        StringParser("||", BinaryOperatorSegment),
+        StringParser("XOR", BinaryOperatorSegment),
     ),
     NotOperatorGrammar=OneOf(
         StringParser("NOT", KeywordSegment, type="keyword"),
@@ -242,18 +244,31 @@ mysql_dialect.replace(
         ),
         ansi_dialect.get_grammar("Expression_C_Grammar"),
     ),
+    ColumnConstraintDefaultGrammar=OneOf(
+        Bracketed(ansi_dialect.get_grammar("ColumnConstraintDefaultGrammar")),
+        ansi_dialect.get_grammar("ColumnConstraintDefaultGrammar"),
+    ),
+    NakedIdentifierSegment=SegmentGenerator(
+        lambda dialect: RegexParser(
+            r"([A-Z0-9_]*[A-Z][A-Z0-9_]*)|_",
+            IdentifierSegment,
+            type="naked_identifier",
+            anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
+        )
+    ),
 )
 
 mysql_dialect.add(
     DoubleQuotedLiteralSegment=TypedParser(
         "double_quote",
-        ansi.LiteralSegment,
+        LiteralSegment,
         type="quoted_literal",
         trim_chars=('"',),
     ),
     AtSignLiteralSegment=TypedParser(
         "at_sign_literal",
-        ansi.LiteralSegment,
+        LiteralSegment,
+        type="at_sign_literal",
     ),
     SystemVariableSegment=RegexParser(
         r"@@(session|global)\.[A-Za-z0-9_]+",
@@ -307,38 +322,36 @@ class ColumnDefinitionSegment(BaseSegment):
             ),
             Sequence(
                 OneOf("DATETIME", "TIMESTAMP"),
-                Sequence(
-                    Bracketed(Ref("NumericLiteralSegment"), optional=True),
-                    optional=True,
-                ),
-                Sequence(Sequence("NOT", optional=True), "NULL", optional=True),
-                Sequence(
-                    "DEFAULT",
-                    OneOf(
-                        Sequence(
-                            OneOf("CURRENT_TIMESTAMP", "NOW"),
+                Bracketed(Ref("NumericLiteralSegment"), optional=True),  # Precision
+                AnyNumberOf(
+                    # Allow NULL/NOT NULL, DEFAULT, and ON UPDATE in any order
+                    Sequence(Sequence("NOT", optional=True), "NULL", optional=True),
+                    Sequence(
+                        "DEFAULT",
+                        OneOf(
+                            Sequence(
+                                OneOf("CURRENT_TIMESTAMP", "NOW"),
+                                Bracketed(
+                                    Ref("NumericLiteralSegment", optional=True),
+                                    optional=True,
+                                ),
+                            ),
+                            Ref("NumericLiteralSegment"),
+                            Ref("QuotedLiteralSegment"),
+                        ),
+                        optional=True,
+                    ),
+                    Sequence(
+                        "ON",
+                        "UPDATE",
+                        OneOf(
+                            "CURRENT_TIMESTAMP",
+                            "NOW",
                             Bracketed(
                                 Ref("NumericLiteralSegment", optional=True),
                                 optional=True,
                             ),
                         ),
-                        Ref("NumericLiteralSegment"),
-                        Ref("QuotedLiteralSegment"),
-                        optional=True,
-                    ),
-                    optional=True,
-                ),
-                Sequence(
-                    Sequence("ON", "UPDATE", optional=True),
-                    Sequence(
-                        OneOf("CURRENT_TIMESTAMP", "NOW"),
-                        Bracketed(
-                            Ref("NumericLiteralSegment", optional=True),
-                            optional=True,
-                        ),
-                    ),
-                    Sequence(
-                        Bracketed(Ref("NumericLiteralSegment")),
                         optional=True,
                     ),
                     optional=True,
@@ -610,6 +623,19 @@ class InsertStatementSegment(BaseSegment):
     )
 
 
+class DeleteTargetTableSegment(BaseSegment):
+    """A target table used in `DELETE` statement.
+
+    https://dev.mysql.com/doc/refman/8.0/en/delete.html
+    """
+
+    type = "delete_target_table"
+    match_grammar = Sequence(
+        Ref("TableReferenceSegment"),
+        Sequence(Ref("DotSegment"), Ref("StarSegment"), optional=True),
+    )
+
+
 class DeleteUsingClauseSegment(BaseSegment):
     """A `USING` clause froma `DELETE` Statement`."""
 
@@ -637,12 +663,18 @@ class DeleteStatementSegment(BaseSegment):
         OneOf(
             Sequence(
                 "FROM",
-                Delimited(Ref("TableReferenceSegment"), terminators=["USING"]),
+                Delimited(
+                    Ref("DeleteTargetTableSegment"),
+                    terminators=["USING"],
+                ),
                 Ref("DeleteUsingClauseSegment"),
                 Ref("WhereClauseSegment", optional=True),
             ),
             Sequence(
-                Delimited(Ref("TableReferenceSegment"), terminators=["FROM"]),
+                Delimited(
+                    Ref("DeleteTargetTableSegment"),
+                    terminators=["FROM"],
+                ),
                 Ref("FromClauseSegment"),
                 Ref("WhereClauseSegment", optional=True),
             ),
@@ -933,7 +965,7 @@ mysql_dialect.insert_lexer_matchers(
             segment_kwargs={"type": "at_sign_literal", "trim_chars": ("@",)},
         ),
     ],
-    before="code",
+    before="word",
 )
 
 
@@ -1070,8 +1102,7 @@ class DeclareStatement(BaseSegment):
 class StatementSegment(ansi.StatementSegment):
     """Overriding StatementSegment to allow for additional segment parsing."""
 
-    match_grammar = ansi.StatementSegment.match_grammar
-    parse_grammar = ansi.StatementSegment.parse_grammar.copy(
+    match_grammar = ansi.StatementSegment.match_grammar.copy(
         insert=[
             Ref("DelimiterStatement"),
             Ref("CreateProcedureStatementSegment"),
@@ -1110,6 +1141,7 @@ class StatementSegment(ansi.StatementSegment):
             Ref("ReplaceSegment"),
             Ref("AlterDatabaseStatementSegment"),
             Ref("ReturnStatementSegment"),
+            Ref("SetNamesStatementSegment"),
         ],
         remove=[
             # handle CREATE SCHEMA in CreateDatabaseStatementSegment
@@ -1226,9 +1258,8 @@ class AlterTableStatementSegment(BaseSegment):
                     Ref("IfNotExistsGrammar", optional=True),
                     Ref("ColumnDefinitionSegment"),
                     OneOf(
-                        Sequence(
-                            OneOf("FIRST", "AFTER"), Ref("ColumnReferenceSegment")
-                        ),
+                        "FIRST",
+                        Sequence("AFTER", Ref("ColumnReferenceSegment")),
                         # Bracketed Version of the same
                         Ref("BracketedColumnReferenceListGrammar"),
                         optional=True,
@@ -1239,9 +1270,8 @@ class AlterTableStatementSegment(BaseSegment):
                     Ref.keyword("COLUMN", optional=True),
                     Ref("ColumnDefinitionSegment"),
                     OneOf(
-                        Sequence(
-                            OneOf("FIRST", "AFTER"), Ref("ColumnReferenceSegment")
-                        ),
+                        "FIRST",
+                        Sequence("AFTER", Ref("ColumnReferenceSegment")),
                         # Bracketed Version of the same
                         Ref("BracketedColumnReferenceListGrammar"),
                         optional=True,
@@ -1577,11 +1607,7 @@ class IntoClauseSegment(BaseSegment):
 
     type = "into_clause"
 
-    match_grammar = StartsWith(
-        "INTO", terminators=[Ref("SelectClauseElementTerminatorGrammar")]
-    )
-
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "INTO",
         OneOf(
             Delimited(
@@ -1632,6 +1658,8 @@ class IntoClauseSegment(BaseSegment):
                 ),
             ),
         ),
+        parse_mode=ParseMode.GREEDY_ONCE_STARTED,
+        terminators=[Ref("SelectClauseTerminatorGrammar")],
     )
 
 
@@ -1644,18 +1672,9 @@ class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
     """
 
     type = "select_statement"
-    match_grammar = ansi.UnorderedSelectStatementSegment.match_grammar.copy(
-        terminators=[
-            Ref("IntoClauseSegment"),
-            Ref("ForClauseSegment"),
-            Ref("IndexHintClauseSegment"),
-            Ref("SelectPartitionClauseSegment"),
-            Ref("UpsertClauseListSegment"),
-        ],
-    )
 
-    parse_grammar = (
-        ansi.UnorderedSelectStatementSegment.parse_grammar.copy(
+    match_grammar = (
+        ansi.UnorderedSelectStatementSegment.match_grammar.copy(
             insert=[Ref("IntoClauseSegment", optional=True)],
             before=Ref("FromClauseSegment", optional=True),
         )
@@ -1667,23 +1686,23 @@ class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
         .copy(
             insert=[Ref("SelectPartitionClauseSegment", optional=True)],
             before=Ref("WhereClauseSegment", optional=True),
+            terminators=[
+                Ref("IntoClauseSegment"),
+                Ref("ForClauseSegment"),
+                Ref("IndexHintClauseSegment"),
+                Ref("SelectPartitionClauseSegment"),
+                Ref("UpsertClauseListSegment"),
+            ],
         )
     )
 
 
-class SelectClauseElementSegment(ansi.SelectClauseElementSegment):
-    """An element in the targets of a select statement."""
-
-    pass
-
-
 class SelectClauseSegment(ansi.SelectClauseSegment):
     """A group of elements in a select target statement."""
 
     match_grammar = ansi.SelectClauseSegment.match_grammar.copy(
         terminators=[Ref("IntoKeywordSegment")],
     )
-    parse_grammar = ansi.SelectClauseSegment.parse_grammar
 
 
 class SelectStatementSegment(ansi.SelectStatementSegment):
@@ -1692,21 +1711,22 @@ class SelectStatementSegment(ansi.SelectStatementSegment):
     https://dev.mysql.com/doc/refman/5.7/en/select.html
     """
 
-    match_grammar = ansi.SelectStatementSegment.match_grammar.copy(
-        terminators=[
-            Ref("UpsertClauseListSegment"),
-            Ref("WithCheckOptionSegment"),
-        ],
-    )
-
     # Inherit most of the parse grammar from the original.
-    parse_grammar = UnorderedSelectStatementSegment.parse_grammar.copy(
+    match_grammar = UnorderedSelectStatementSegment.match_grammar.copy(
         insert=[
             Ref("OrderByClauseSegment", optional=True),
             Ref("LimitClauseSegment", optional=True),
             Ref("NamedWindowSegment", optional=True),
             Ref("IntoClauseSegment", optional=True),
-        ]
+        ],
+        terminators=[
+            Ref("SetOperatorSegment"),
+            Ref("UpsertClauseListSegment"),
+            Ref("WithCheckOptionSegment"),
+        ],
+        # Overwrite the terminators, because we want to remove some from the
+        # expression above.
+        replace_terminators=True,
     )
 
 
@@ -1755,26 +1775,11 @@ class CallStoredProcedureSegment(BaseSegment):
     https://dev.mysql.com/doc/refman/8.0/en/call.html
     """
 
-    type = "call_segment"
+    type = "call_statement"
 
     match_grammar = Sequence(
         "CALL",
-        OneOf(
-            Ref("SingleIdentifierGrammar"),
-            Ref("QuotedIdentifierSegment"),
-        ),
-        Bracketed(
-            AnyNumberOf(
-                Delimited(
-                    Ref("QuotedLiteralSegment"),
-                    Ref("NumericLiteralSegment"),
-                    Ref("DoubleQuotedLiteralSegment"),
-                    Ref("SessionVariableNameSegment"),
-                    Ref("LocalVariableNameSegment"),
-                    Ref("FunctionSegment"),
-                ),
-            ),
-        ),
+        Ref("FunctionSegment"),
     )
 
 
@@ -2625,7 +2630,7 @@ class CreateOptionSegment(BaseSegment):
                 "CHARACTER",
                 "SET",
                 Ref("EqualsSegment", optional=True),
-                Ref("NakedIdentifierSegment"),
+                OneOf(Ref("NakedIdentifierSegment"), Ref("QuotedLiteralSegment")),
             ),
             Sequence(
                 "COLLATE",
@@ -2705,3 +2710,18 @@ class ReturnStatementSegment(BaseSegment):
         "RETURN",
         Ref("ExpressionSegment"),
     )
+
+
+class SetNamesStatementSegment(BaseSegment):
+    """A `SET NAMES` statement.
+
+    As specified in https://dev.mysql.com/doc/refman/8.0/en/set-names.html
+    """
+
+    type = "set_names_statement"
+    match_grammar: Matchable = Sequence(
+        "SET",
+        "NAMES",
+        OneOf("DEFAULT", Ref("QuotedLiteralSegment"), Ref("NakedIdentifierSegment")),
+        Sequence("COLLATE", Ref("CollationReferenceSegment"), optional=True),
+    )
