@@ -2,6 +2,9 @@
 
 This inherits from the ansi dialect.
 """
+
+from typing import cast
+
 from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
     AnyNumberOf,
@@ -12,11 +15,15 @@ from sqlfluff.core.parser import (
     BracketedSegment,
     CodeSegment,
     CommentSegment,
+    CompositeComparisonOperatorSegment,
     Delimited,
-    GreedyUntil,
+    IdentifierSegment,
+    LiteralSegment,
     Matchable,
+    Nothing,
     OneOf,
     OptionallyBracketed,
+    ParseMode,
     Ref,
     RegexLexer,
     RegexParser,
@@ -25,6 +32,8 @@ from sqlfluff.core.parser import (
     StringLexer,
     StringParser,
     SymbolSegment,
+    TypedParser,
+    WordSegment,
 )
 from sqlfluff.dialects import dialect_ansi as ansi
 
@@ -48,11 +57,14 @@ oracle_dialect.sets("reserved_keywords").update(
         "SIBLINGS",
         "START",
         "CONNECT_BY_ROOT",
+        "PIVOT",
+        "FOR",
+        "UNPIVOT",
     ]
 )
 
 oracle_dialect.sets("unreserved_keywords").update(
-    ["EDITIONABLE", "EDITIONING", "NONEDITIONABLE"]
+    ["EDITIONABLE", "EDITIONING", "NONEDITIONABLE", "KEEP"]
 )
 
 oracle_dialect.sets("bare_functions").clear()
@@ -71,11 +83,11 @@ oracle_dialect.sets("bare_functions").update(
 
 oracle_dialect.patch_lexer_matchers(
     [
+        RegexLexer("word", r"[a-zA-Z][0-9a-zA-Z_$#]*", WordSegment),
         RegexLexer(
-            "code",
-            r"[a-zA-Z][0-9a-zA-Z_$#]*",
+            "single_quote",
+            r"'([^'\\]|\\|\\.|'')*'",
             CodeSegment,
-            segment_kwargs={"type": "code"},
         ),
     ]
 )
@@ -89,7 +101,7 @@ oracle_dialect.insert_lexer_matchers(
         ),
         StringLexer("at_sign", "@", CodeSegment),
     ],
-    before="code",
+    before="word",
 )
 
 oracle_dialect.insert_lexer_matchers(
@@ -112,6 +124,38 @@ oracle_dialect.add(
         ),
     ),
     ConnectByRootGrammar=Sequence("CONNECT_BY_ROOT", Ref("NakedIdentifierSegment")),
+    PlusJoinSegment=Bracketed(
+        StringParser("+", SymbolSegment, type="plus_join_symbol")
+    ),
+    PlusJoinGrammar=OneOf(
+        Sequence(
+            OneOf(Ref("ColumnReferenceSegment"), Ref("FunctionSegment")),
+            Ref("EqualsSegment"),
+            Ref("ColumnReferenceSegment"),
+            Ref("PlusJoinSegment"),
+        ),
+        Sequence(
+            Ref("ColumnReferenceSegment"),
+            Ref("PlusJoinSegment"),
+            Ref("EqualsSegment"),
+            OneOf(Ref("ColumnReferenceSegment"), Ref("FunctionSegment")),
+        ),
+    ),
+    IntervalUnitsGrammar=OneOf("YEAR", "MONTH", "DAY", "HOUR", "MINUTE", "SECOND"),
+    PivotForInGrammar=Sequence(
+        "FOR",
+        OptionallyBracketed(Delimited(Ref("ColumnReferenceSegment"))),
+        "IN",
+        Bracketed(
+            Delimited(
+                Sequence(
+                    Ref("Expression_D_Grammar"),
+                    Ref("AliasExpressionSegment", optional=True),
+                )
+            )
+        ),
+    ),
+    UnpivotNullsGrammar=Sequence(OneOf("INCLUDE", "EXCLUDE"), "NULLS"),
 )
 
 oracle_dialect.replace(
@@ -128,7 +172,7 @@ oracle_dialect.replace(
     NakedIdentifierSegment=SegmentGenerator(
         lambda dialect: RegexParser(
             r"[A-Z0-9_]*[A-Z][A-Z0-9_#$]*",
-            ansi.IdentifierSegment,
+            IdentifierSegment,
             type="naked_identifier",
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
         )
@@ -168,6 +212,79 @@ oracle_dialect.replace(
             Ref("ConnectByRootGrammar"),
         ]
     ),
+    Expression_D_Grammar=Sequence(
+        OneOf(
+            Ref("PlusJoinGrammar"),
+            Ref("BareFunctionSegment"),
+            Ref("FunctionSegment"),
+            Bracketed(
+                OneOf(
+                    # We're using the expression segment here rather than the grammar so
+                    # that in the parsed structure we get nested elements.
+                    Ref("ExpressionSegment"),
+                    Ref("SelectableGrammar"),
+                    Delimited(
+                        Ref(
+                            "ColumnReferenceSegment"
+                        ),  # WHERE (a,b,c) IN (select a,b,c FROM...)
+                        Ref(
+                            "FunctionSegment"
+                        ),  # WHERE (a, substr(b,1,3)) IN (select c,d FROM...)
+                        Ref("LiteralGrammar"),  # WHERE (a, 2) IN (SELECT b, c FROM ...)
+                        Ref("LocalAliasSegment"),  # WHERE (LOCAL.a, LOCAL.b) IN (...)
+                    ),
+                ),
+                parse_mode=ParseMode.GREEDY,
+            ),
+            # Allow potential select statement without brackets
+            Ref("SelectStatementSegment"),
+            Ref("LiteralGrammar"),
+            Ref("IntervalExpressionSegment"),
+            Ref("TypedStructLiteralSegment"),
+            Ref("ArrayExpressionSegment"),
+            Ref("ColumnReferenceSegment"),
+            # For triggers, we allow "NEW.*" but not just "*" nor "a.b.*"
+            # So can't use WildcardIdentifierSegment nor WildcardExpressionSegment
+            Sequence(
+                Ref("SingleIdentifierGrammar"),
+                Ref("ObjectReferenceDelimiterGrammar"),
+                Ref("StarSegment"),
+            ),
+            Sequence(
+                Ref("StructTypeSegment"),
+                Bracketed(Delimited(Ref("ExpressionSegment"))),
+            ),
+            Sequence(
+                Ref("DatatypeSegment"),
+                # Don't use the full LiteralGrammar here
+                # because only some of them are applicable.
+                # Notably we shouldn't use QualifiedNumericLiteralSegment
+                # here because it looks like an arithmetic operation.
+                OneOf(
+                    Ref("QuotedLiteralSegment"),
+                    Ref("NumericLiteralSegment"),
+                    Ref("BooleanLiteralGrammar"),
+                    Ref("NullLiteralSegment"),
+                    Ref("DateTimeLiteralGrammar"),
+                ),
+            ),
+            Ref("LocalAliasSegment"),
+            terminators=[Ref("CommaSegment")],
+        ),
+        Ref("AccessorGrammar", optional=True),
+        allow_gaps=True,
+    ),
+    DateTimeLiteralGrammar=Sequence(
+        OneOf("DATE", "TIME", "TIMESTAMP", "INTERVAL"),
+        TypedParser("single_quote", LiteralSegment, type="date_constructor_literal"),
+        Sequence(
+            Ref("IntervalUnitsGrammar"),
+            Sequence("TO", Ref("IntervalUnitsGrammar"), optional=True),
+        ),
+    ),
+    PreTableFunctionKeywordsGrammar=OneOf("LATERAL"),
+    ConditionalCrossJoinKeywordsGrammar=Nothing(),
+    UnconditionalCrossJoinKeywordsGrammar=Ref.keyword("CROSS"),
 )
 
 
@@ -265,7 +382,7 @@ class AlterTableColumnClausesSegment(BaseSegment):
             Ref("ColumnReferenceSegment"),
             "TO",
             Ref("ColumnReferenceSegment"),
-        )
+        ),
         # @TODO: modify_collection_retrieval
         # @TODO: modify_LOB_storage_clause
         # @TODO: alter_varray_col_properties
@@ -358,10 +475,7 @@ class StatementSegment(ansi.StatementSegment):
 
     type = "statement"
 
-    match_grammar = OneOf(
-        GreedyUntil(Ref("DelimiterGrammar")), exclude=Ref("ExecuteFileSegment")
-    )
-    parse_grammar = ansi.StatementSegment.parse_grammar.copy(
+    match_grammar = ansi.StatementSegment.match_grammar.copy(
         insert=[
             Ref("CommentStatementSegment"),
         ],
@@ -379,9 +493,7 @@ class FileSegment(BaseFileSegment):
     ending in DelimiterGrammar
     """
 
-    # NB: We don't need a match_grammar here because we're
-    # going straight into instantiating it directly usually.
-    parse_grammar = AnyNumberOf(
+    match_grammar = AnyNumberOf(
         Ref("ExecuteFileSegment"),
         Delimited(
             Ref("StatementSegment"),
@@ -433,18 +545,10 @@ class CommentStatementSegment(BaseSegment):
     )
 
 
-# Inherit from the ANSI ObjectReferenceSegment this way so we can inherit
-# other segment types from it.
-class ObjectReferenceSegment(ansi.ObjectReferenceSegment):
-    """A reference to an object."""
-
-    pass
-
-
 # need to ignore type due to mypy rules on type variables
 # see https://mypy.readthedocs.io/en/stable/common_issues.html#variables-vs-type-aliases
 # for details
-class TableReferenceSegment(ObjectReferenceSegment):
+class TableReferenceSegment(ansi.ObjectReferenceSegment):
     """A reference to an table, CTE, subquery or alias.
 
     Extended from ANSI to allow Database Link syntax using AtSignSegment
@@ -604,7 +708,18 @@ class ColumnDefinitionSegment(BaseSegment):
             ),
             Sequence(
                 Ref("DatatypeSegment"),  # Column type
-                Bracketed(Anything(), optional=True),  # For types like VARCHAR(100)
+                # For types like VARCHAR(100), VARCHAR(100 BYTE), VARCHAR (100 CHAR)
+                Bracketed(
+                    Sequence(
+                        Anything(),
+                        OneOf(
+                            "BYTE",
+                            "CHAR",
+                            optional=True,
+                        ),
+                    ),
+                    optional=True,
+                ),
                 AnyNumberOf(
                     Ref("ColumnConstraintSegment", optional=True),
                 ),
@@ -661,7 +776,7 @@ class StartWithClauseSegment(BaseSegment):
 
 
 class HierarchicalQueryClauseSegment(BaseSegment):
-    """Hiearchical Query.
+    """Hierarchical Query.
 
     https://docs.oracle.com/en/database/oracle/oracle-database/21/sqlrf/Hierarchical-Queries.html
     """
@@ -697,29 +812,38 @@ class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
     """
 
     match_grammar = ansi.UnorderedSelectStatementSegment.match_grammar.copy(
-        terminators=[Ref("HierarchicalQueryClauseSegment")],
-    )
-    parse_grammar: Matchable = ansi.UnorderedSelectStatementSegment.parse_grammar.copy(
-        insert=[Ref("HierarchicalQueryClauseSegment", optional=True)],
+        insert=[
+            Ref("HierarchicalQueryClauseSegment", optional=True),
+            Ref("PivotSegment", optional=True),
+            Ref("UnpivotSegment", optional=True),
+        ],
         before=Ref("GroupByClauseSegment", optional=True),
+        terminators=[
+            Ref("HierarchicalQueryClauseSegment"),
+            Ref("PivotSegment", optional=True),
+            Ref("UnpivotSegment", optional=True),
+        ],
     )
 
 
 class SelectStatementSegment(ansi.SelectStatementSegment):
     """A `SELECT` statement."""
 
-    match_grammar: Matchable = ansi.SelectStatementSegment.match_grammar.copy()
-    parse_grammar: Matchable = UnorderedSelectStatementSegment.parse_grammar.copy(
+    match_grammar: Matchable = UnorderedSelectStatementSegment.match_grammar.copy(
         insert=[
             Ref("OrderByClauseSegment", optional=True),
             Ref("FetchClauseSegment", optional=True),
             Ref("LimitClauseSegment", optional=True),
             Ref("NamedWindowSegment", optional=True),
-        ]
+        ],
+        replace_terminators=True,
+        terminators=cast(
+            Sequence, ansi.SelectStatementSegment.match_grammar
+        ).terminators,
     )
 
 
-class GreaterThanOrEqualToSegment(ansi.CompositeComparisonOperatorSegment):
+class GreaterThanOrEqualToSegment(CompositeComparisonOperatorSegment):
     """Allow spaces between operators."""
 
     match_grammar = OneOf(
@@ -734,7 +858,7 @@ class GreaterThanOrEqualToSegment(ansi.CompositeComparisonOperatorSegment):
     )
 
 
-class LessThanOrEqualToSegment(ansi.CompositeComparisonOperatorSegment):
+class LessThanOrEqualToSegment(CompositeComparisonOperatorSegment):
     """Allow spaces between operators."""
 
     match_grammar = OneOf(
@@ -749,10 +873,92 @@ class LessThanOrEqualToSegment(ansi.CompositeComparisonOperatorSegment):
     )
 
 
-class NotEqualToSegment(ansi.CompositeComparisonOperatorSegment):
+class NotEqualToSegment(CompositeComparisonOperatorSegment):
     """Allow spaces between operators."""
 
     match_grammar = OneOf(
         Sequence(Ref("RawNotSegment"), Ref("RawEqualsSegment")),
         Sequence(Ref("RawLessThanSegment"), Ref("RawGreaterThanSegment")),
     )
+
+
+class PivotSegment(BaseSegment):
+    """Pivot clause.
+
+    https://docs.oracle.com/en/database/oracle/oracle-database/21/sqlrf/SELECT.html
+    """
+
+    type = "pivot_clause"
+
+    match_grammar: Matchable = Sequence(
+        "PIVOT",
+        Ref.keyword("XML", optional=True),
+        Bracketed(
+            Delimited(
+                Ref("FunctionSegment"), Ref("AliasExpressionSegment", optional=True)
+            ),
+            Ref("PivotForInGrammar"),
+        ),
+    )
+
+
+class UnpivotSegment(BaseSegment):
+    """Unpivot clause.
+
+    https://docs.oracle.com/en/database/oracle/oracle-database/21/sqlrf/SELECT.html
+    """
+
+    type = "unpivot_clause"
+
+    match_grammar: Matchable = Sequence(
+        "UNPIVOT",
+        Ref("UnpivotNullsGrammar", optional=True),
+        Bracketed(
+            OptionallyBracketed(Delimited(Ref("ColumnReferenceSegment"))),
+            Ref("PivotForInGrammar"),
+        ),
+    )
+
+
+class ObjectReferenceSegment(ansi.ObjectReferenceSegment):
+    """A reference to an object."""
+
+    # Allow whitespace
+    match_grammar: Matchable = Delimited(
+        Ref("SingleIdentifierGrammar"),
+        delimiter=Ref("ObjectReferenceDelimiterGrammar"),
+        terminators=[Ref("ObjectReferenceTerminatorGrammar")],
+        allow_gaps=True,
+    )
+
+
+class ColumnReferenceSegment(ObjectReferenceSegment):
+    """A reference to column, field or alias."""
+
+    type = "column_reference"
+
+
+class FunctionNameSegment(BaseSegment):
+    """Function name, including any prefix bits, e.g. project or schema."""
+
+    type = "function_name"
+    match_grammar: Matchable = Sequence(
+        # Project name, schema identifier, etc.
+        AnyNumberOf(
+            Sequence(
+                Ref("SingleIdentifierGrammar"),
+                Ref("DotSegment"),
+            ),
+            terminators=[Ref("BracketedSegment")],
+        ),
+        # Base function name
+        Delimited(
+            OneOf(
+                Ref("FunctionNameIdentifierSegment"),
+                Ref("QuotedIdentifierSegment"),
+                terminators=[Ref("BracketedSegment")],
+            ),
+            delimiter=Ref("AtSignSegment"),
+        ),
+        allow_gaps=False,
+    )
