@@ -3,6 +3,7 @@
 This is based on postgres dialect, since it was initially based off of Postgres 8.
 We should monitor in future and see if it should be rebased off of ANSI
 """
+
 from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
     AnyNumberOf,
@@ -10,20 +11,21 @@ from sqlfluff.core.parser import (
     Anything,
     BaseSegment,
     Bracketed,
-    CodeSegment,
     Dedent,
     Delimited,
+    IdentifierSegment,
     Indent,
     Matchable,
     Nothing,
     OneOf,
     OptionallyBracketed,
+    ParseMode,
     Ref,
     RegexLexer,
     RegexParser,
     SegmentGenerator,
     Sequence,
-    StartsWith,
+    WordSegment,
 )
 from sqlfluff.dialects import dialect_ansi as ansi
 from sqlfluff.dialects import dialect_postgres as postgres
@@ -185,7 +187,7 @@ redshift_dialect.replace(
             # must only contain digits, letters, underscore, and $ but
             # can’t be all digits.
             r"#?([A-Z_]+|[0-9]+[A-Z_$])[A-Z0-9_$]*",
-            ansi.IdentifierSegment,
+            IdentifierSegment,
             type="naked_identifier",
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
         )
@@ -194,25 +196,16 @@ redshift_dialect.replace(
 
 redshift_dialect.patch_lexer_matchers(
     [
-        # add optional leading # to code for temporary tables
+        # add optional leading # to word for temporary tables
         RegexLexer(
-            "code",
+            "word",
             r"#?[0-9a-zA-Z_]+[0-9a-zA-Z_$]*",
-            CodeSegment,
-            segment_kwargs={"type": "code"},
+            WordSegment,
         ),
     ]
 )
 
 
-# Inherit from the Postgres ObjectReferenceSegment this way so we can inherit
-# other segment types from it.
-class ObjectReferenceSegment(postgres.ObjectReferenceSegment):
-    """A reference to an object."""
-
-    pass
-
-
 redshift_dialect.add(
     CompressionTypeGrammar=OneOf(
         "BZIP2",
@@ -694,6 +687,13 @@ class AlterTableActionSegment(BaseSegment):
             Ref("ColumnReferenceSegment"),
             Ref("DropBehaviorGrammar", optional=True),
         ),
+        Sequence(
+            "APPEND",
+            "FROM",
+            Ref("TableReferenceSegment"),
+            Ref.keyword("IGNOREEXTRA", optional=True),
+            Ref.keyword("FILLTARGET", optional=True),
+        ),
     )
 
 
@@ -748,7 +748,7 @@ class TableConstraintSegment(BaseSegment):
                 Bracketed(Delimited(Ref("ColumnReferenceSegment"))),
                 "REFERENCES",
                 Ref("TableReferenceSegment"),
-                Sequence(Bracketed(Ref("ColumnReferenceSegment"))),
+                Bracketed(Delimited(Ref("ColumnReferenceSegment"))),
             ),
         ),
     )
@@ -1843,7 +1843,7 @@ class GrantUsageDatashareStatementSegment(BaseSegment):
 
     https://docs.aws.amazon.com/redshift/latest/dg/r_GRANT.html
     section "Granting datashare permissions"
-    Note: According to docummentation, multiple accounts and namespaces can be
+    Note: According to documentation, multiple accounts and namespaces can be
           specified. However, tests using redshift instance showed this causes a syntax
           error.
     """
@@ -2024,8 +2024,7 @@ class StatementSegment(postgres.StatementSegment):
 
     type = "statement"
 
-    match_grammar = postgres.StatementSegment.match_grammar
-    parse_grammar = postgres.StatementSegment.parse_grammar.copy(
+    match_grammar = postgres.StatementSegment.match_grammar.copy(
         insert=[
             Ref("CreateLibraryStatementSegment"),
             Ref("CreateGroupStatementSegment"),
@@ -2489,9 +2488,9 @@ class FunctionSegment(ansi.FunctionSegment):
                             "FunctionContentsGrammar",
                             # The brackets might be empty for some functions...
                             optional=True,
-                            ephemeral_name="FunctionContentsGrammar",
                         ),
-                    )
+                    ),
+                    parse_mode=ParseMode.GREEDY,
                 ),
             ),
         ),
@@ -2523,8 +2522,8 @@ class FunctionSegment(ansi.FunctionSegment):
                         "FunctionContentsGrammar",
                         # The brackets might be empty for some functions...
                         optional=True,
-                        ephemeral_name="FunctionContentsGrammar",
-                    )
+                    ),
+                    parse_mode=ParseMode.GREEDY,
                 ),
             ),
             Ref("PostFunctionGrammar", optional=True),
@@ -2656,19 +2655,11 @@ class SelectStatementSegment(postgres.SelectStatementSegment):
     """
 
     type = "select_statement"
-    match_grammar = StartsWith(
-        # NB: In bigquery, the select clause may include an EXCEPT, which
-        # will also match the set operator, but by starting with the whole
-        # select clause rather than just the SELECT keyword, we normally
-        # mitigate that here. But this isn't BigQuery! So we can be more
-        # efficient and just use the keyword.
-        "SELECT",
-        terminators=[Ref("SetOperatorSegment")],
-    )
 
-    parse_grammar = postgres.SelectStatementSegment.parse_grammar.copy(
+    match_grammar = postgres.SelectStatementSegment.match_grammar.copy(
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OrderByClauseSegment", optional=True),
+        terminators=[Ref("SetOperatorSegment")],
     )
 
 
@@ -2679,9 +2670,8 @@ class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
     """
 
     type = "select_statement"
-    match_grammar = ansi.UnorderedSelectStatementSegment.match_grammar.copy()
 
-    parse_grammar = ansi.UnorderedSelectStatementSegment.parse_grammar.copy(
+    match_grammar = ansi.UnorderedSelectStatementSegment.match_grammar.copy(
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OverlapsClauseSegment", optional=True),
     )
