@@ -8,6 +8,7 @@ ALTER
 AND
 ANY
 AS
+ASOF
 BETWEEN
 BY
 CAST
@@ -49,6 +50,7 @@ LEFT
 LIKE
 LOCALTIME
 LOCALTIMESTAMP
+MATCH_CONDITION
 MATCH_RECOGNIZE
 MINUS
 NATURAL
@@ -99,9 +101,11 @@ ADD
 ADMIN
 AFTER
 ALLOWED_IP_LIST
+ALLOWED_NETWORK_RULE_LIST
 ALLOWED_VALUES
 ALLOW_DUPLICATE
 ALLOW_OVERLAPPING_EXECUTION
+ALLOW_WRITES
 API
 API_INTEGRATION
 APPEND_ONLY
@@ -133,6 +137,7 @@ AZURE
 AZURE_EVENT_GRID
 AZURE_EVENT_GRID_TOPIC_ENDPOINT
 AZURE_SAS_TOKEN
+AZURE_STORAGE_QUEUE
 AZURE_STORAGE_QUEUE_PRIMARY_URI
 AZURE_TENANT_ID
 BASE64
@@ -145,6 +150,8 @@ BINARY_FORMAT
 BINDING
 BLOCK
 BLOCKED_IP_LIST
+BLOCKED_NETWORK_RULE_LIST
+BODY
 BROTLI
 BZ2
 CACHE
@@ -176,6 +183,7 @@ CONTINUE
 COPY
 COPY_OPTIONS
 CREDENTIALS
+CREDIT_QUOTA
 CROSS
 CSV
 CUBE
@@ -192,6 +200,7 @@ CURRENT_STATEMENT
 CURRENT_TRANSACTION
 CURRENT_VERSION
 CURRENT_WAREHOUSE
+CURSOR
 CYCLE
 DATA
 DATABASE
@@ -200,6 +209,7 @@ DATA_FORMAT
 DATA_RETENTION_TIME_IN_DAYS
 DATE
 DATE_FORMAT
+DAILY
 DAYS_TO_EXPIRY
 DEFAULT
 DEFAULT_DDL_COLLATION
@@ -222,8 +232,10 @@ DISABLED
 DISABLE_AUTO_CONVERT
 DISABLE_SNOWFLAKE_DATA
 DISPLAY_NAME
+DO
 DOMAIN
 DOUBLE
+DYNAMIC
 ECONOMY
 EMAIL
 EMPTY
@@ -234,6 +246,7 @@ ENABLE_OCTAL
 ENCODING
 ENCRYPTION
 END
+END_TIMESTAMP
 ENFORCED
 ENFORCE_LENGTH
 ENUM
@@ -261,11 +274,13 @@ FILTER
 FINAL
 FIRST
 FIRST_NAME
+FOR
 FORCE
 FOREIGN
 FORMAT
 FORMATS
 FORMAT_NAME
+FREQUENCY
 FUNCTION
 FUNCTIONS
 FUTURE
@@ -290,6 +305,7 @@ IF
 IGNORE
 IGNORE_UTF8_ERRORS
 IMMEDIATE
+IMMEDIATELY
 IMMUTABLE
 IMPORT
 IMPORTS
@@ -298,6 +314,7 @@ INCLUDE_QUERY_ID
 INDEX
 INFORMATION
 INITIALLY
+INITIALIZE
 INITIALLY_SUSPENDED
 INPUT
 INTEGRATION
@@ -315,6 +332,7 @@ LAST
 LAST_NAME
 LAST_QUERY_ID
 LAST_TRANSACTION
+LET
 LIMIT
 LIST
 LISTING
@@ -325,6 +343,7 @@ LOGIN_NAME
 LS
 LZO
 M
+MAIN_FILE
 MANAGE
 MANAGED
 MASKING
@@ -353,12 +372,14 @@ MODEL
 MODIFIED_AFTER
 MODIFY
 MONITOR
+MONTHLY
 MUST_CHANGE_PASSWORD
 NAME
 NAN
 NETWORK
 NEXT
 NEXTVAL
+NEVER
 NO
 NOCACHE
 NOCYCLE
@@ -367,6 +388,8 @@ NOORDER
 NOTIFICATION
 NOTIFICATION_INTEGRATION
 NOTIFICATION_PROVIDER
+NOTIFY
+NOTIFY_USERS
 NULLS
 NULL_IF
 OBJECT
@@ -400,6 +423,7 @@ PAST
 PATTERN
 FIELD_OPTIONALITY_ENCLOSED_BY
 PER
+PERCENT
 PERMUTE
 PIPE
 PIPES
@@ -421,6 +445,7 @@ PURGE
 PUT
 PYTHON
 QUERIES
+QUERY_WAREHOUSE
 QUEUE
 RANGE
 RAW_DEFLATE
@@ -431,6 +456,7 @@ RECURSIVE
 REFERENCES
 REFERENCE_USAGE
 REFRESH
+REFRESH_MODE
 REFRESH_ON_CREATE
 REGIONS
 REMOVE
@@ -448,7 +474,9 @@ RESPONSE_TRANSLATOR
 RESTRICT
 RESTRICTIONS
 RESULT
+RESULTSET
 RESUME
+RETURN
 RETURNS
 RETURN_ALL_ERRORS
 RETURN_ERRORS
@@ -458,6 +486,7 @@ ROLE
 ROLES
 ROLLBACK
 ROLLUP
+ROOT_LOCATION
 ROUTINE
 ROUTINES
 ROW
@@ -507,24 +536,32 @@ STAGE_COPY_OPTIONS
 STAGE_FILE_FORMAT
 STANDARD
 STARTS
+START_TIMESTAMP
 STATEMENT
 STATEMENT_QUEUED_TIMEOUT_IN_SECONDS
 STATEMENT_TIMEOUT_IN_SECONDS
 STORAGE
 STORAGE_ALLOWED_LOCATIONS
+STORAGE_AWS_EXTERNAL_ID
 STORAGE_AWS_OBJECT_ACL
 STORAGE_AWS_ROLE_ARN
+STORAGE_BASE_URL
 STORAGE_BLOCKED_LOCATIONS
 STORAGE_INTEGRATION
+STORAGE_LOCATION
+STORAGE_LOCATIONS
 STORAGE_PROVIDER
 STREAM
 STREAMS
+STREAMLIT
+STREAMLITS
 STRIP_NULL_VALUES
 STRIP_OUTER_ARRAY
 STRIP_OUTER_ELEMENT
 SUBPATH
 SUPPORT
 SUSPEND
+SUSPEND_IMMEDIATE
 SUSPENDED
 SWAP
 SYSDATE
@@ -534,6 +571,7 @@ TABLESPACE
 TABULAR
 TAG
 TARGET_PATH
+TARGET_LAG
 TASK
 TASKS
 TEMP
@@ -550,6 +588,7 @@ TRANSACTION
 TRANSACTIONS
 TRANSIENT
 TRIM_SPACE
+TRIGGERS
 TRUNCATE
 TRUNCATECOLUMNS
 TYPE
@@ -578,11 +617,14 @@ VERSION
 VIEW
 VIEWS
 VOLATILE
+VOLUME
+VOLUMES
 WAIT_FOR_COMPLETION
 WAREHOUSE
 WAREHOUSES
 WAREHOUSE_SIZE
 WAREHOUSE_TYPE
+WEEKLY
 WINDOW
 WITH
 WITHIN
@@ -592,6 +634,7 @@ WOY
 WRAPPER
 WRITE
 XML
+YEARLY
 ZONE
 ZSTD
 """
