@@ -5,19 +5,24 @@ https://www.sqlite.org/
 
 from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
-    BaseSegment,
-    Bracketed,
-    Matchable,
-    OneOf,
-    OptionallyBracketed,
-    Ref,
-    Sequence,
-    Delimited,
-    TypedParser,
-    Nothing,
     AnyNumberOf,
     Anything,
-    Dedent,
+    BaseSegment,
+    Bracketed,
+    CommentSegment,
+    Delimited,
+    LiteralSegment,
+    Matchable,
+    NewlineSegment,
+    Nothing,
+    OneOf,
+    OptionallyBracketed,
+    ParseMode,
+    Ref,
+    RegexLexer,
+    Sequence,
+    TypedParser,
+    WhitespaceSegment,
 )
 from sqlfluff.dialects import dialect_ansi as ansi
 from sqlfluff.dialects.dialect_sqlite_keywords import (
@@ -34,19 +39,35 @@ sqlite_dialect.sets("reserved_keywords").update(RESERVED_KEYWORDS)
 sqlite_dialect.sets("unreserved_keywords").clear()
 sqlite_dialect.sets("unreserved_keywords").update(UNRESERVED_KEYWORDS)
 
+sqlite_dialect.patch_lexer_matchers(
+    [
+        # SQLite allows block comments to be terminated by end of input
+        RegexLexer(
+            "block_comment",
+            r"\/\*([^\*]|\*(?!\/))*(\*\/|\Z)",
+            CommentSegment,
+            subdivider=RegexLexer(
+                "newline",
+                r"\r\n|\n",
+                NewlineSegment,
+            ),
+            trim_post_subdivide=RegexLexer(
+                "whitespace",
+                r"[^\S\r\n]+",
+                WhitespaceSegment,
+            ),
+        ),
+    ]
+)
+
 sqlite_dialect.replace(
-    BooleanBinaryOperatorGrammar=OneOf(
-        Ref("AndOperatorGrammar"), Ref("OrOperatorGrammar"), "REGEXP"
-    ),
     PrimaryKeyGrammar=Sequence(
         "PRIMARY", "KEY", Sequence("AUTOINCREMENT", optional=True)
     ),
     TemporaryTransientGrammar=Ref("TemporaryGrammar"),
     DateTimeLiteralGrammar=Sequence(
         OneOf("DATE", "DATETIME"),
-        TypedParser(
-            "single_quote", ansi.LiteralSegment, type="date_constructor_literal"
-        ),
+        TypedParser("single_quote", LiteralSegment, type="date_constructor_literal"),
     ),
     BaseExpressionElementGrammar=OneOf(
         Ref("LiteralGrammar"),
@@ -100,7 +121,7 @@ sqlite_dialect.replace(
     ),
     PostFunctionGrammar=Ref("FilterClauseGrammar"),
     IgnoreRespectNullsGrammar=Nothing(),
-    SelectClauseElementTerminatorGrammar=OneOf(
+    SelectClauseTerminatorGrammar=OneOf(
         "FROM",
         "WHERE",
         Sequence("ORDER", "BY"),
@@ -164,9 +185,15 @@ sqlite_dialect.replace(
         Ref("TildeSegment"),
         Ref("NotOperatorGrammar"),
     ),
-    IsClauseGrammar=OneOf(
-        "NULL",
-        Ref("BooleanLiteralGrammar"),
+    IsDistinctFromGrammar=Sequence(
+        "IS",
+        Ref.keyword("NOT", optional=True),
+        Sequence("DISTINCT", "FROM", optional=True),
+    ),
+    NanLiteralSegment=Nothing(),
+    PatternMatchingGrammar=Sequence(
+        Ref.keyword("NOT", optional=True),
+        OneOf("GLOB", "REGEXP", "MATCH"),
     ),
 )
 
@@ -236,8 +263,8 @@ class ValuesClauseSegment(ansi.ValuesClauseSegment):
                     Delimited(
                         "DEFAULT",
                         Ref("ExpressionSegment"),
-                        ephemeral_name="ValuesClauseElements",
-                    )
+                    ),
+                    parse_mode=ParseMode.GREEDY,
                 ),
             ),
         ),
@@ -261,6 +288,26 @@ class IndexColumnDefinitionSegment(BaseSegment):
     )
 
 
+class ReturningClauseSegment(BaseSegment):
+    """A returning clause.
+
+    Per docs https://www.sqlite.org/lang_returning.html
+    """
+
+    type = "returning_clause"
+
+    match_grammar = Sequence(
+        "RETURNING",
+        Delimited(
+            Ref("WildcardExpressionSegment"),
+            Sequence(
+                Ref("ExpressionSegment"),
+                Ref("AliasExpressionSegment", optional=True),
+            ),
+        ),
+    )
+
+
 class InsertStatementSegment(BaseSegment):
     """An`INSERT` statement.
 
@@ -295,6 +342,7 @@ class InsertStatementSegment(BaseSegment):
             OptionallyBracketed(Ref("SelectableGrammar")),
             Ref("DefaultValuesGrammar"),
         ),
+        Ref("ReturningClauseSegment", optional=True),
     )
 
 
@@ -464,30 +512,16 @@ class CreateTriggerStatementSegment(ansi.CreateTriggerStatementSegment):
     )
 
 
-class SelectClauseSegment(BaseSegment):
-    """A group of elements in a select target statement.
-
-    Overriding ANSI to remove StartsWith logic which assumes statements have been
-    delimited
-    """
-
-    type = "select_clause"
-    match_grammar = Ref("SelectClauseSegmentGrammar")
-
-
 class UnorderedSelectStatementSegment(BaseSegment):
     """A `SELECT` statement without any ORDER clauses or later.
 
-    Replaces (without overriding) ANSI to remove Greedy Matcher
+    Replaces (without overriding) ANSI to remove Eager Matcher
     """
 
     type = "select_statement"
 
     match_grammar = Sequence(
         Ref("SelectClauseSegment"),
-        # Dedent for the indent in the select clause.
-        # It's here so that it can come AFTER any whitespace.
-        Dedent,
         Ref("FromClauseSegment", optional=True),
         Ref("WhereClauseSegment", optional=True),
         Ref("GroupByClauseSegment", optional=True),
@@ -497,10 +531,47 @@ class UnorderedSelectStatementSegment(BaseSegment):
     )
 
 
+class DeleteStatementSegment(ansi.DeleteStatementSegment):
+    """A `DELETE` statement.
+
+    DELETE FROM <table name> [ WHERE <search condition> ]
+    """
+
+    type = "delete_statement"
+    # match grammar. This one makes sense in the context of knowing that it's
+    # definitely a statement, we just don't know what type yet.
+    match_grammar: Matchable = Sequence(
+        "DELETE",
+        Ref("FromClauseSegment"),
+        Ref("WhereClauseSegment", optional=True),
+        Ref("ReturningClauseSegment", optional=True),
+    )
+
+
+class UpdateStatementSegment(ansi.UpdateStatementSegment):
+    """An `Update` statement.
+
+    UPDATE <table name> SET <set clause list> [ WHERE <search condition> ]
+    """
+
+    type = "update_statement"
+    match_grammar: Matchable = Sequence(
+        "UPDATE",
+        Ref("TableReferenceSegment"),
+        # SET is not a reserved word in all dialects (e.g. RedShift)
+        # So specifically exclude as an allowed implicit alias to avoid parsing errors
+        Ref("AliasExpressionSegment", exclude=Ref.keyword("SET"), optional=True),
+        Ref("SetClauseListSegment"),
+        Ref("FromClauseSegment", optional=True),
+        Ref("WhereClauseSegment", optional=True),
+        Ref("ReturningClauseSegment", optional=True),
+    )
+
+
 class SelectStatementSegment(BaseSegment):
     """A `SELECT` statement.
 
-    Replaces (without overriding) ANSI to remove Greedy Matcher
+    Replaces (without overriding) ANSI to remove Eager Matcher
     """
 
     type = "select_statement"
@@ -563,5 +634,3 @@ class StatementSegment(ansi.StatementSegment):
         Ref("UpdateStatementSegment"),
         Bracketed(Ref("StatementSegment")),
     )
-
-    parse_grammar = match_grammar
