@@ -1,17 +1,18 @@
 """Static methods to support ReflowPoint.respace_point()."""
 
-
 import logging
-from typing import List, Optional, Tuple, cast, TYPE_CHECKING
+from typing import TYPE_CHECKING, Dict, List, Optional, Tuple, cast
 
-from sqlfluff.core.parser import BaseSegment, RawSegment, PositionMarker
-from sqlfluff.core.parser.segments.raw import WhitespaceSegment
-from sqlfluff.core.rules.base import LintFix, LintResult
 from sqlfluff.core.errors import SQLFluffUserError
-
+from sqlfluff.core.parser import (
+    BaseSegment,
+    PositionMarker,
+    RawSegment,
+    WhitespaceSegment,
+)
+from sqlfluff.core.rules import LintFix, LintResult
 from sqlfluff.utils.reflow.helpers import pretty_segment_name
 
-
 if TYPE_CHECKING:  # pragma: no cover
     from sqlfluff.utils.reflow.elements import ReflowBlock
 
@@ -91,6 +92,9 @@ def determine_constraints(
             pre_constraint = "touch"
         if post_constraint != "any":
             post_constraint = "touch"
+    elif within_spacing == "any":
+        pre_constraint = "any"
+        post_constraint = "any"
     elif within_spacing == "single":
         pass
     elif within_spacing:  # pragma: no cover
@@ -214,20 +218,41 @@ def _determine_aligned_inline_spacing(
     # We've got a parent. Find some siblings.
     reflow_logger.debug("    Determining alignment within: %s", parent_segment)
     siblings = []
-    if align_scope:
-        for sibling in parent_segment.recursive_crawl(segment_type):
-            # Purge any siblings with a boundary between them
-            if not any(
-                ps.segment.is_type(align_scope)
-                for ps in parent_segment.path_to(sibling)
-            ):
-                siblings.append(sibling)
-            else:
-                reflow_logger.debug(
-                    "    Purging a sibling because they're blocked "
-                    "by a boundary: %s",
-                    sibling,
-                )
+    for sibling in parent_segment.recursive_crawl(segment_type):
+        # Purge any siblings with a boundary between them
+        if not align_scope or not any(
+            ps.segment.is_type(align_scope) for ps in parent_segment.path_to(sibling)
+        ):
+            siblings.append(sibling)
+        else:
+            reflow_logger.debug(
+                "    Purging a sibling because they're blocked " "by a boundary: %s",
+                sibling,
+            )
+
+    # If the segment we're aligning, has position. Use that position.
+    # If it doesn't, then use the provided one. We can't do sibling analysis without it.
+    if next_seg.pos_marker:
+        next_pos = next_seg.pos_marker
+
+    # Purge any siblings which are either self, or on the same line but after it.
+    _earliest_siblings: Dict[int, int] = {}
+    for sibling in siblings[:]:
+        _pos = sibling.pos_marker
+        assert _pos
+        _best_seen = _earliest_siblings.get(_pos.working_line_no, None)
+        # If we've already seen an earlier sibling on this line, ignore the later one.
+        if _best_seen is not None and _pos.working_line_pos > _best_seen:
+            siblings.remove(sibling)
+            continue
+        # Update best seen
+        _earliest_siblings[_pos.working_line_no] = _pos.working_line_pos
+
+        # We should also purge the sibling which matches the target.
+        if _pos.working_line_no == next_pos.working_line_no:
+            # Is it in the same position?
+            if _pos.working_line_pos != next_pos.working_line_pos:
+                siblings.remove(sibling)
 
     # If there's only one sibling, we have nothing to compare to. Default to a single
     # space.
@@ -239,23 +264,6 @@ def _determine_aligned_inline_spacing(
         )
         return desired_space
 
-    # If the segment we're aligning, has position. Use that position.
-    # If it doesn't, then use the provided one. We can't do sibling analysis without it.
-    if next_seg.pos_marker:
-        next_pos = next_seg.pos_marker
-
-    # Is the current indent the only one on the line?
-    if any(
-        sibling.pos_marker
-        # Same line
-        and sibling.pos_marker.working_line_no == next_pos.working_line_no
-        # And not same position (i.e. not self)
-        and sibling.pos_marker.working_line_pos != next_pos.working_line_pos
-        for sibling in siblings
-    ):
-        reflow_logger.debug("    Found sibling on same line. Treat as single")
-        return " "
-
     # Work out the current spacing before each.
     last_code = None
     max_desired_line_pos = 0
